@@ -1,103 +1,131 @@
 ## --- New "logic" class -- currently using "raw" instead of "logical"
 ## LOGIC setClass("logic", contains = "raw")
 
+##' To be used in initialize method or other Matrix constructors
+##'
+##' TODO: via .Call(..)
+.fixupDimnames <- function(dnms) {
+    N.N <- list(NULL, NULL)
+    if(is.null(dnms) || identical(dnms, N.N)) return(N.N)
+    ## else
+    if(any(i0 <- lengths(dnms) == 0) && !all(vapply(dnms[i0], is.null, NA)))
+	## replace character(0) etc, by NULL :
+	dnms[i0] <- list(NULL)
+    ## coerce, e.g. integer dimnames to character: -- as  R's matrix(..):
+    if(any(i0 <- vapply(dnms, function(d) !is.null(d) && !is.character(d), NA)))
+	dnms[i0] <- lapply(dnms[i0], as.character)
+    dnms
+}
+
+
 ## ------------- Virtual Classes ----------------------------------------
 
 ## Mother class of all Matrix objects
-setClass("Matrix",
-	 representation(Dim = "integer", Dimnames = "list", "VIRTUAL"),
+setClass("Matrix", contains = "VIRTUAL",
+	 slots = c(Dim = "integer", Dimnames = "list"),
 	 prototype = prototype(Dim = integer(2), Dimnames = list(NULL,NULL)),
 	 validity = function(object) {
-	     Dim <- object@Dim
-	     if (length(Dim) != 2)
-		 return("Dim slot must be of length 2")
-	     if (any(Dim < 0))
-		 return("Dim slot must contain non-negative values")
-	     Dn <- object@Dimnames
-	     if (!is.list(Dn) || length(Dn) != 2)
-		 return("'Dimnames' slot must be list of length 2")
-	     lDn <- sapply(Dn, length)
-	     if (lDn[1] > 0 && lDn[1] != Dim[1])
-		 return("length(Dimnames[[1]])' must match Dim[1]")
-	     if (lDn[2] > 0 && lDn[2] != Dim[2])
-		 return("length(Dimnames[[2]])' must match Dim[2]")
-	     ## 'else'	ok :
-	     TRUE
+	     if(!isTRUE(r <- .Call(Dim_validate, object, "Matrix")))
+                 r
+             else .Call(dimNames_validate, object)
 	 })
 
+if(FALSE)## Allowing 'Dimnames' to define 'Dim' --> would require changes in
+    ##  ../src/Mutils.c dimNames_validate() and how it is used in validity above
+setMethod("initialize", "Matrix", function(.Object, ...)
+    {
+        .Object <- callNextMethod()
+	if(length(args <- list(...)) && any(nzchar(snames <- names(args))) && "Dimnames" %in% snames)
+	{
+	    .Object@Dimnames <- DN <- .fixupDimnames(.Object@Dimnames)
+	    if(is.na(match("Dim", snames)) && !any(vapply(DN, is.null, NA)))
+		## take 'Dim' from 'Dimnames' dimensions
+		.Object@Dim <- lengths(DN, use.names=FALSE)
+	}
+	.Object
+    })
+
+if(getRversion() >= "3.2.0") {
+setMethod("initialize", "Matrix", function(.Object, ...)
+    {
+        .Object <- callNextMethod()
+	if(length(args <- list(...)) && any(nzchar(snames <- names(args))) && "Dimnames" %in% snames)
+	    .Object@Dimnames <- .fixupDimnames(.Object@Dimnames)
+	.Object
+    })
+} else { ## R < 3.2.0
+setMethod("initialize", "Matrix", function(.Object, ...)
+    {
+	.Object <- callNextMethod(.Object, ...)
+	if(length(args <- list(...)) && any(nzchar(snames <- names(args))) && "Dimnames" %in% snames)
+	    .Object@Dimnames <- .fixupDimnames(.Object@Dimnames)
+	.Object
+    })
+}
+
 ## The class of composite matrices - i.e. those for which it makes sense to
 ## create a factorization
-setClass("compMatrix",	representation(factors = "list", "VIRTUAL"),
-	 contains = "Matrix")
+setClass("compMatrix", contains = c("Matrix", "VIRTUAL"),
+	 slots = c(factors = "list"))
 
 ## Virtual classes of Matrices determined by above/below diagonal relationships
 
-setClass("generalMatrix", representation = "VIRTUAL", contains = "compMatrix")
+setClass("generalMatrix", contains = c("compMatrix", "VIRTUAL"))
 
-setClass("symmetricMatrix",
-	 representation(uplo = "character", "VIRTUAL"),
-	 contains = "compMatrix",
+setClass("symmetricMatrix", contains = c("compMatrix", "VIRTUAL"),
+	 slots = c(uplo = "character"),
 	 prototype = prototype(uplo = "U"),
 	 validity = function(object) .Call(symmetricMatrix_validate, object))
 
-setClass("triangularMatrix",
-	 representation(uplo = "character", diag = "character", "VIRTUAL"),
-	 contains = "Matrix",
+setClass("triangularMatrix", contains = c("Matrix", "VIRTUAL"),
+	 slots = c(uplo = "character", diag = "character"),
 	 prototype = prototype(uplo = "U", diag = "N"),
 	 validity = function(object) .Call(triangularMatrix_validate, object))
 
 
 ## Virtual class of numeric matrices
-setClass("dMatrix",
-	 representation(x = "numeric", "VIRTUAL"), contains = "Matrix",
+setClass("dMatrix", contains = c("Matrix", "VIRTUAL"), slots = c(x = "numeric"),
 	 validity = function(object) .Call(dMatrix_validate, object))
 
 ## Virtual class of integer matrices
-setClass("iMatrix",
-	 representation(x = "integer", "VIRTUAL"), contains = "Matrix")
+setClass("iMatrix", contains = c("Matrix", "VIRTUAL"), slots = c(x = "integer"))
 
 ## Virtual class of logical matrices
-setClass("lMatrix",
-## LOGIC representation(x = "logic", "VIRTUAL"), contains = "Matrix")
-         representation(x = "logical", "VIRTUAL"), contains = "Matrix")
+setClass("lMatrix", contains = c("Matrix", "VIRTUAL"), slots = c(x = "logical"))
 
 ## Virtual class of nonzero pattern matrices
-setClass("nMatrix", representation("VIRTUAL"), contains = "Matrix")
+setClass("nMatrix", contains = c("Matrix", "VIRTUAL"))
 ## aka 'pattern' matrices -- have no x slot
 
-## Virtual class of complex matrices
-setClass("zMatrix", # letter 'z' is as in the names of Lapack subroutines
-	 representation(x = "complex", "VIRTUAL"), contains = "Matrix")
+## Virtual class of complex matrices - 'z'  as in the names of Lapack routines
+setClass("zMatrix", contains = c("Matrix", "VIRTUAL"), slots = c(x = "complex"))
 
 ## Virtual class of dense matrices (including "packed")
-setClass("denseMatrix", representation("VIRTUAL"),
-	 contains = "Matrix")
+setClass("denseMatrix", contains = c("Matrix", "VIRTUAL"))
 
 ## Virtual class of dense, numeric matrices
-setClass("ddenseMatrix", representation("VIRTUAL"),
-	 contains = c("dMatrix", "denseMatrix"))
+setClass("ddenseMatrix", contains = c("dMatrix", "denseMatrix", "VIRTUAL"))
 
 ## Virtual class of dense, logical matrices
-setClass("ldenseMatrix", representation("VIRTUAL"),
-	 contains = c("lMatrix", "denseMatrix"))
+setClass("ldenseMatrix", contains = c("lMatrix", "denseMatrix", "VIRTUAL"))
 
 if(FALSE) { ##--not yet--
-setClass("idenseMatrix", representation("VIRTUAL"),
-	 contains = c("iMatrix", "denseMatrix"))
+setClass("idenseMatrix", contains = c("iMatrix", "denseMatrix", "VIRTUAL"))
 }
 
 ## Virtual class of dense, nonzero pattern matrices - rarely used, for completeness
-setClass("ndenseMatrix", representation(x = "logical", "VIRTUAL"),
-	 contains = c("nMatrix", "denseMatrix"))
+setClass("ndenseMatrix", contains = c("nMatrix", "denseMatrix", "VIRTUAL"),
+	 slots = c(x = "logical"))
+
 
 ## virtual SPARSE ------------
 
-setClass("sparseMatrix", representation("VIRTUAL"), contains = "Matrix")
+setClass("sparseMatrix", contains = c("Matrix", "VIRTUAL"))
 
 ## diagonal: has 'diag' slot;  diag = "U"  <--> have identity matrix
-setClass("diagonalMatrix", representation(diag = "character", "VIRTUAL"),
-	 contains = "sparseMatrix",
-         ## NOTE:    ^^^^^^ was dense Matrix, until 0.999375-11 (2008-07)
+setClass("diagonalMatrix", contains = c("sparseMatrix", "VIRTUAL"),
+         ## NOTE:                        ^^^^^^ was dense Matrix, until 0.999375-11 (2008-07)
+         slots = c(diag = "character"),
 	 validity = function(object) {
 	     d <- object@Dim
 	     if(d[1] != (n <- d[2])) return("matrix is not square")
@@ -115,49 +143,63 @@ setClass("diagonalMatrix", representation(diag = "character", "VIRTUAL"),
 	 )
 
 ## sparse matrices in Triplet representation (dgT, lgT, ..):
-setClass("TsparseMatrix", representation(i = "integer", j = "integer", "VIRTUAL"),
-	 contains = "sparseMatrix",
+setClass("TsparseMatrix", contains = c("sparseMatrix", "VIRTUAL"),
+	 slots = c(i = "integer", j = "integer"),
 	 validity = function(object) .Call(Tsparse_validate, object)
          )
 
-setClass("CsparseMatrix", representation(i = "integer", p = "integer", "VIRTUAL"),
-	 contains = "sparseMatrix",
+setClass("CsparseMatrix", contains = c("sparseMatrix", "VIRTUAL"),
+	 slots = c(i = "integer", p = "integer"),
 	 prototype = prototype(p = 0L),# to be valid
          validity = function(object) .Call(Csparse_validate, object)
          )
 
-setClass("RsparseMatrix", representation(p = "integer", j = "integer", "VIRTUAL"),
-	 contains = "sparseMatrix",
+if(FALSE) { ## in theory.. would be neat for  new("dgCMatrix", Dim = c(3L,3L))
+setMethod("initialize", "CsparseMatrix", function(.Object, ...) {
+    .Object <- callNextMethod()
+    .Object@p <- integer(.Object@Dim[2L] + 1L)
+    .Object
+})
+
+setMethod("initialize", "RsparseMatrix", function(.Object, ...) {
+    .Object <- callNextMethod()
+    .Object@p <- integer(.Object@Dim[1L] + 1L)
+    .Object
+})
+}# not yet (fails)
+
+setClass("RsparseMatrix", contains = c("sparseMatrix", "VIRTUAL"),
+	 slots = c(p = "integer", j = "integer"),
 	 prototype = prototype(p = 0L),# to be valid
 	 validity = function(object) .Call(Rsparse_validate, object)
          )
 
-setClass("dsparseMatrix", representation("VIRTUAL"),
-	 contains = c("dMatrix", "sparseMatrix"))
+setClass("dsparseMatrix", contains = c("dMatrix", "sparseMatrix", "VIRTUAL"))
 
-setClass("lsparseMatrix", representation("VIRTUAL"),
-	 contains = c("lMatrix", "sparseMatrix"))
+setClass("lsparseMatrix", contains = c("lMatrix", "sparseMatrix", "VIRTUAL"))
 
 if(FALSE) { ##--not yet--
-setClass("isparseMatrix", representation("VIRTUAL"),
-	 contains = c("lMatrix", "sparseMatrix"))
+setClass("isparseMatrix", contains = c("iMatrix", "sparseMatrix", "VIRTUAL"))
 }
 
 ## these are the "pattern" matrices for "symbolic analysis" of sparse OPs:
-setClass("nsparseMatrix", representation("VIRTUAL"),
-	 contains = c("nMatrix", "sparseMatrix"))
+setClass("nsparseMatrix", contains = c("nMatrix", "sparseMatrix", "VIRTUAL"))
 
-if(FALSE) { ##-- a nice idea --- but needs more careful method re-definitions
-            ##-- such that the *correct* methods are dispatched:
-## Trying to use more economical method defs:
-setClass("dCsparseMatrix", representation("VIRTUAL"),
-	 contains = c("CsparseMatrix", "dsparseMatrix"))
-setClass("lCsparseMatrix", representation("VIRTUAL"),
-	 contains = c("CsparseMatrix", "lsparseMatrix"))
-setClass("nCsparseMatrix", representation("VIRTUAL"),
-	 contains = c("CsparseMatrix", "nsparseMatrix"))
+## More Class Intersections {for method dispatch}:
+if(FALSE) { ## this is "natural" but gives WARNINGs when other packages use "it"
+setClass("dCsparseMatrix", contains = c("CsparseMatrix", "dsparseMatrix", "VIRTUAL"))
+setClass("lCsparseMatrix", contains = c("CsparseMatrix", "lsparseMatrix", "VIRTUAL"))
+setClass("nCsparseMatrix", contains = c("CsparseMatrix", "nsparseMatrix", "VIRTUAL"))
+
+## dense general
+setClass("geMatrix", contains = c("denseMatrix", "generalMatrix", "VIRTUAL"))
+
+} else { ## ----------- a version that maybe works better for other pkgs ---------
+
+ ##--> setClassUnion() ... below
 }
 
+
 ## ------------------ Proper (non-virtual) Classes ----------------------------
 
 ##----------------------  DENSE	 -----------------------------------------
@@ -165,9 +207,7 @@ setClass("nCsparseMatrix", representation("VIRTUAL"),
 ## numeric, dense, general matrices
 setClass("dgeMatrix", contains = c("ddenseMatrix", "generalMatrix"),
 	 ## checks that length( @ x) == prod( @ Dim):
-	 validity =
-	 function(object) .Call(dgeMatrix_validate, object)
-	 )
+	 validity = function(object) .Call(dgeMatrix_validate, object))
 ## i.e. "dgeMatrix" cannot be packed, but "ddenseMatrix" can ..
 
 ## numeric, dense, non-packed, triangular matrices
@@ -178,9 +218,8 @@ setClass("dtrMatrix",
 ## numeric, dense, packed, triangular matrices
 setClass("dtpMatrix",
 	 contains = c("ddenseMatrix", "triangularMatrix"),
-	 validity =
-	 function(object) .Call(dtpMatrix_validate, object)
-	 )
+	 validity = function(object) .Call(dtpMatrix_validate, object))
+
 
 ## numeric, dense, non-packed symmetric matrices
 setClass("dsyMatrix",
@@ -190,9 +229,7 @@ setClass("dsyMatrix",
 ## numeric, dense, packed symmetric matrices
 setClass("dspMatrix",
 	 contains = c("ddenseMatrix", "symmetricMatrix"),
-	 validity =
-	 function(object) .Call(dspMatrix_validate, object)
-	 )
+	 validity = function(object) .Call(dspMatrix_validate, object))
 
 ## numeric, dense, non-packed, positive-definite, symmetric matrices
 setClass("dpoMatrix", contains = "dsyMatrix",
@@ -265,11 +302,11 @@ setClass("nspMatrix",
 
 ## 'diagonalMatrix' already has validity checking
 ## diagonal, numeric matrices; "dMatrix" has 'x' slot :
-setClass("ddiMatrix", contains = c("diagonalMatrix", "dMatrix"))# or "dMatrix"
+setClass("ddiMatrix", contains = c("diagonalMatrix", "dMatrix"))
 ## diagonal, logical matrices; "lMatrix" has 'x' slot :
 setClass("ldiMatrix", contains = c("diagonalMatrix", "lMatrix"))
 
-setClass("corMatrix", representation(sd = "numeric"), contains = "dpoMatrix",
+setClass("corMatrix", slots = c(sd = "numeric"), contains = "dpoMatrix",
 	 validity = function(object) {
 	     ## assuming that 'dpoMatrix' validity check has already happened:
 	     n <- object@Dim[2]
@@ -317,8 +354,7 @@ setClass("dgCMatrix",
 	 )
 
 ## special case: indicator rows for a factor - needs more careful definition
-#setClass("indicators", representation(levels = "character"),
-#	 contains = "dgCMatrix")
+##setClass("indicators", contains = "dgCMatrix", slots = c(levels = "character"))
 
 ## see comments for dtTMatrix above
 ## numeric, sparse, sorted compressed sparse column-oriented triangular matrices
@@ -537,20 +573,20 @@ setClass("isRMatrix",
 
 ##-------------------- index and permutation matrices--------------------------
 
-setClass("indMatrix", representation(perm = "integer"),
+setClass("indMatrix", slots = c(perm = "integer"),
 	 contains = c("sparseMatrix", "generalMatrix"),
 	 validity = function(object) {
-	     n <-  object@Dim[1]
-	     d <-  object@Dim[2]
+	     n <- object@Dim[1]
+	     d <- object@Dim[2]
 	     perm <- object@perm
 	     if (length(perm) != n)
 		 return(paste("length of 'perm' slot must be", n))
-	     if(n > 0 && (any(perm > d) | any(perm < 1)))
+	     if(n > 0 && (any(perm > d) || any(perm < 1)))
 		 return("'perm' slot is not a valid index")
 	     TRUE
 	 })
 
-setClass("pMatrix", representation(perm = "integer"),
+setClass("pMatrix", slots = c(perm = "integer"),
 	 contains = c("indMatrix"),
 	 validity = function(object) {
 	     d <- object@Dim
@@ -568,10 +604,10 @@ setClass("pMatrix", representation(perm = "integer"),
 ### Factorization classes ---------------------------------------------
 
 ## Mother class:
-setClass("MatrixFactorization", representation(Dim = "integer", "VIRTUAL"))
+setClass("MatrixFactorization", slots = c(Dim = "integer"), contains = "VIRTUAL",
+	 validity = function(object) .Call(MatrixFactorization_validate, object))
 
-setClass("CholeskyFactorization", representation = "VIRTUAL",
-         contains = "MatrixFactorization")
+setClass("CholeskyFactorization", contains = "MatrixFactorization", "VIRTUAL")
 
 ## -- Those (exceptions) inheriting from "Matrix" : ---
 
@@ -582,64 +618,58 @@ setClass("Cholesky",  contains = c("dtrMatrix", "CholeskyFactorization"))
 setClass("pCholesky", contains = c("dtpMatrix", "CholeskyFactorization"))
 
 ## These are currently only produced implicitly from *solve()
-setClass("BunchKaufman",
-	 contains = c("dtrMatrix", "MatrixFactorization"),
-	 representation(perm = "integer"),
-	 validity =
-	 function(object) .Call(BunchKaufman_validate, object)
-	 )
+setClass("BunchKaufman", contains = c("dtrMatrix", "MatrixFactorization"),
+	 slots = c(perm = "integer"),
+	 validity = function(object) .Call(BunchKaufman_validate, object))
 
-setClass("pBunchKaufman",
-	 contains = c("dtpMatrix", "MatrixFactorization"),
-	 representation(perm = "integer"),
-	 validity =
-	 function(object) .Call(pBunchKaufman_validate, object))
+setClass("pBunchKaufman", contains = c("dtpMatrix", "MatrixFactorization"),
+	 slots = c(perm = "integer"),
+	 validity = function(object) .Call(pBunchKaufman_validate, object))
 
 ## -- the usual ``non-Matrix'' factorizations : ---------
 
-setClass("CHMfactor",		 # cholmod_factor struct as S4 object
-	 contains = "CholeskyFactorization",
-	 representation(colcount = "integer", perm = "integer",
-			type = "integer", "VIRTUAL"),
+setClass("CHMfactor", # cholmod_factor struct as S4 object
+	 contains = c("CholeskyFactorization", "VIRTUAL"),
+	 slots = c(colcount = "integer", perm = "integer", type = "integer"),
 	 validity = function(object) .Call(CHMfactor_validate, object))
 
 setClass("CHMsuper",		       # supernodal cholmod_factor
-	 contains = "CHMfactor",
-	 representation(super = "integer", pi = "integer", px = "integer",
-			s = "integer", "VIRTUAL"),
+	 contains = c("CHMfactor", "VIRTUAL"),
+	 slots = c(super = "integer", pi = "integer", px = "integer",
+		   s = "integer"),
 	 validity = function(object) .Call(CHMsuper_validate, object))
 
 setClass("CHMsimpl",		       # simplicial cholmod_factor
-	 contains = "CHMfactor",
-	 representation(p = "integer", i = "integer", nz = "integer",
-			nxt = "integer", prv = "integer", "VIRTUAL"),
+	 contains = c("CHMfactor", "VIRTUAL"),
+	 slots = c(p = "integer", i = "integer", nz = "integer",
+		   nxt = "integer", prv = "integer"),
 	 validity = function(object) .Call(CHMsimpl_validate, object))
 
-setClass("dCHMsuper", contains = "CHMsuper", representation(x = "numeric"))
+setClass("dCHMsuper", contains = "CHMsuper", slots = c(x = "numeric"))
 
 setClass("nCHMsuper", contains = "CHMsuper")
 
-setClass("dCHMsimpl", contains = "CHMsimpl", representation(x = "numeric"))
+setClass("dCHMsimpl", contains = "CHMsimpl", slots = c(x = "numeric"))
 
 setClass("nCHMsimpl", contains = "CHMsimpl")
 
 ##--- LU ---
 
-setClass("LU", contains = "MatrixFactorization", representation("VIRTUAL"))
+setClass("LU", contains = c("MatrixFactorization", "VIRTUAL"))
 
 setClass("denseLU", contains = "LU",
-	 representation(x = "numeric", perm = "integer"),
+	 slots = c(x = "numeric", perm = "integer", Dimnames = "list"),
 	 validity = function(object) .Call(LU_validate, object))
 
 setClass("sparseLU", contains = "LU",
-	 representation(L = "dtCMatrix", U = "dtCMatrix",
-			p = "integer", q = "integer"))
+	 slots = c(L = "dtCMatrix", U = "dtCMatrix",
+		   p = "integer", q = "integer"))
 
 ##--- QR ---
 
 setClass("sparseQR", contains = "MatrixFactorization",
-	 representation(V = "dgCMatrix", beta = "numeric",
-			p = "integer", R = "dgCMatrix", q = "integer"),
+	 slots = c(V = "dgCMatrix", beta = "numeric",
+		   p = "integer", R = "dgCMatrix", q = "integer"),
 	 validity = function(object) .Call(sparseQR_validate, object))
 
 ##-- "SPQR" ---> ./spqr.R  for now
@@ -647,17 +677,17 @@ setClass("sparseQR", contains = "MatrixFactorization",
 ## "denseQR" -- ?  (``a version of''  S3 class "qr")
 
 if (FALSE) { ## unused classes
-setClass("csn_QR", representation(U = "dgCMatrix", L = "dgCMatrix",
-                                  beta = "numeric"))
+setClass("csn_QR", slots = c(U = "dgCMatrix", L = "dgCMatrix",
+                             beta = "numeric"))
 
-setClass("csn_LU", representation(U = "dgCMatrix", L = "dgCMatrix",
-                                  Pinv = "integer"))
+setClass("csn_LU", slots = c(U = "dgCMatrix", L = "dgCMatrix",
+                             Pinv = "integer"))
 
-setClass("css_QR", representation(Pinv = "integer", Q = "integer",
-                                  parent = "integer", cp = "integer",
-                                  nz = "integer"))
+setClass("css_QR", slots = c(Pinv = "integer", Q = "integer",
+                             parent = "integer", cp = "integer",
+                             nz = "integer"))
 
-setClass("css_LU", representation(Q = "integer", nz = "integer"))
+setClass("css_LU", slots = c(Q = "integer", nz = "integer"))
 }
 
 ##-- Schur ---
@@ -666,8 +696,8 @@ setClass("css_LU", representation(Q = "integer", nz = "integer"))
 setClassUnion("number", members = c("numeric", "complex"))
 
 setClass("Schur", contains = "MatrixFactorization",
-	 representation(T = "Matrix", # <- "block-triangular"; maybe triangular
-			Q = "Matrix", EValues = "number"),
+	 slots = c(T = "Matrix", # <- "block-triangular"; maybe triangular
+                   Q = "Matrix", EValues = "number"),
 	 validity = function(object) {
 	     dim <- object@Dim
 	     if((n <- dim[1]) != dim[2])
@@ -685,6 +715,8 @@ setClass("Schur", contains = "MatrixFactorization",
 ### Class Union :  no inheritance, but is(*, <class>) :
 
 setClassUnion("mMatrix", members = c("matrix", "Matrix"))
+if(FALSE) ## to be used in setMethod("c", "numM...") -- once that works
+setClassUnion("numMatrixLike", members = c("logical", "integer", "numeric", "mMatrix"))
 
 ## CARE: Sometimes we'd want all those for which 'x' contains all the data.
 ##       e.g. Diagonal() is "ddiMatrix" with 'x' slot of length 0, does *not* contain 1
@@ -695,6 +727,17 @@ setClassUnion("xMatrix", ## those Matrix classes with an 'x' slot
                 "ndenseMatrix",
                 "zMatrix"))
 
+if(TRUE) { ##--- variant of setClass("dCsparse..." ..) etc working better for other pkgs -----
+
+setClassUnion("dCsparseMatrix", members = c("dgCMatrix", "dtCMatrix", "dsCMatrix"))
+setClassUnion("lCsparseMatrix", members = c("lgCMatrix", "ltCMatrix", "lsCMatrix"))
+setClassUnion("nCsparseMatrix", members = c("ngCMatrix", "ntCMatrix", "nsCMatrix"))
+
+## dense general
+setClassUnion("geMatrix", members = c("dgeMatrix", "lgeMatrix", "ngeMatrix"))
+}
+
+
 
 ## Definition  Packed := dense with length( . @x) < prod( . @Dim)
 ##	       ~~~~~~
@@ -708,17 +751,15 @@ setClassUnion("xMatrix", ## those Matrix classes with an 'x' slot
 
 ## --- "General" (not Matrix at all) ----
 
-## e.g. for "Arith" methods:
+## e.g. for "Arith" methods, NB: --> see "numericVector" below (incl "integer")
 setClassUnion("numLike", members = c("numeric", "logical"))
 
 ##setClassUnion("numIndex", members = "numeric")
 
 ## Note "rle" is a sealed oldClass (and "virtual" as w/o prototype)
-setClass("rleDiff", representation(first = "numLike", rle = "rle"),
+setClass("rleDiff", slots = c(first = "numLike", rle = "rle"),
 	 prototype = prototype(first = integer(),
-		     ## workaround buglet in R <= 2.10.x: rle = rle(integer())
-	 rle = structure(list(lengths = integer(0L), values = 0[0L]), class = "rle")
-	 ),
+			       rle = rle(integer())),
 	 validity = function(object) {
 	     if(length(object@first) != 1)
 		 return("'first' must be of length one")
@@ -753,11 +794,11 @@ setClass("seqMat", contains = "matrix",
 	 })
 
 setClass("abIndex", # 'ABSTRact Index'
-         representation(kind = "character",
-                        ## one of ("int32", "double", "rleDiff")
+         slots = c(kind = "character",
+                   ## one of ("int32", "double", "rleDiff")
                                         # i.e., numeric or "rleDiff"
-                        x = "numLike", # for  numeric [length 0 otherwise]
-                        rleD = "rleDiff"),  # "rleDiff" result
+                   x = "numLike", # for  numeric [length 0 otherwise]
+                   rleD = "rleDiff"),  # "rleDiff" result
          prototype = prototype(kind = "int32", x = integer(0)),# rleD = ... etc
          validity = function(object) {
             switch(object@kind,
@@ -787,10 +828,24 @@ setClassUnion("atomicVector", ## "double" is not needed, and not liked by some
 	      members = c("logical", "integer", "numeric",
 			  "complex", "raw", "character"))
 
-## --- Matrix - related (but not "Matrix" nor "Decomposition/Factorization):
+## NB: --> see "numLike" above
+setClassUnion("numericVector", members = c("logical", "integer", "numeric"))
 
-## for 'value' in  x[..] <- value hence for all "contents" of our Matrices:
-setClassUnion("replValue", members = c("numeric", "logical", "complex", "raw"))
+setClassUnion("Mnumeric", members = c("numericVector", "Matrix"))
+## not "matrix" as that extends "vector" and contains "character", "structure" ...
+
+setValidity("Mnumeric",
+            function(object) {
+                if(is.numeric(object) ||
+                   is.logical(object) ||
+                   inherits(object, "Matrix")) return(TRUE)
+                ## else
+                "Not a valid 'Mnumeric' class object"
+		})
+
+
+
+## --- Matrix - related (but not "Matrix" nor "Decomposition/Factorization):
 
 ### Sparse Vectors ---- here use 1-based indexing ! -----------
 
@@ -799,7 +854,7 @@ setClassUnion("replValue", members = c("numeric", "logical", "complex", "raw"))
 ## setClass("longindex", contains = "numeric")
 ## but we use "numeric" instead, for simplicity (efficiency?)
 setClass("sparseVector",
-         representation(length = "numeric", i = "numeric", "VIRTUAL"),
+         slots = c(length = "numeric", i = "numeric"), contains = "VIRTUAL",
          ##                     "longindex"    "longindex"
          ## note that "numeric" contains "integer" (if I like it or not..)
 	 prototype = prototype(length = 0),
@@ -816,10 +871,29 @@ setClass("sparseVector",
          })
 
 ##' initialization -- ensuring that  'i' is sorted (and 'x' alongside)
+if(getRversion() >= "3.2.0") {
+setMethod("initialize", "sparseVector", function(.Object, i, x, ...)
+      {
+	  has.x <- !missing(x)
+	  if(!missing(i)) {
+	      i <- ## (be careful to assign in all cases)
+		  if(is.unsorted(i, strictly=TRUE)) {
+		      if(is(.Object, "xsparseVector") && has.x) {
+			  si <- sort.int(i, index.return=TRUE)
+			  x <- x[si$ix]
+			  si$x
+		      }
+		      else
+			  sort.int(i, method = "quick")
+		  }
+		  else i
+	  }
+	  if(has.x) x <- x
+	  callNextMethod()
+      })
+} else { ## R < 3.2.0
 setMethod("initialize", "sparseVector", function(.Object, i, x, ...)
       {
-	  ## NB: could simplify for R(-devel) >= 2015-01-14 (i.e. >= 3.2.0),
-	  ##	 assigning 'i' and 'x' and calling  callNextMethod() {w/o arg.s}
 	  has.x <- !missing(x)
 	  if(!missing(i)) {
 	      .Object@i <- ## (be careful to assign in all cases)
@@ -837,6 +911,7 @@ setMethod("initialize", "sparseVector", function(.Object, i, x, ...)
 	  if(has.x) .Object@x <- x
 	  callNextMethod(.Object, ...)
       })
+}
 
 .validXspVec <- function(object) {
     ## n <- object@length
@@ -845,16 +920,16 @@ setMethod("initialize", "sparseVector", function(.Object, i, x, ...)
     else TRUE
 }
 setClass("dsparseVector",
-	 representation(x = "numeric"), contains = "sparseVector",
+	 slots = c(x = "numeric"), contains = "sparseVector",
 	 validity = .validXspVec)
 setClass("isparseVector",
-	 representation(x = "integer"), contains = "sparseVector",
+	 slots = c(x = "integer"), contains = "sparseVector",
 	 validity = .validXspVec)
 setClass("lsparseVector",
-	 representation(x = "logical"), contains = "sparseVector",
+	 slots = c(x = "logical"), contains = "sparseVector",
 	 validity = .validXspVec)
 setClass("zsparseVector",
-	 representation(x = "complex"), contains = "sparseVector",
+	 slots = c(x = "complex"), contains = "sparseVector",
 	 validity = .validXspVec)
 ## nsparse has no new slot: 'i' just contains the locations!
 setClass("nsparseVector", contains = "sparseVector")
@@ -865,9 +940,13 @@ setClassUnion("xsparseVector", ## those sparseVector's with an 'x' slot
                 "lsparseVector",
                 "zsparseVector"))
 
+## for 'value' in  x[..] <- value hence for all "contents" of our Matrices:
+setClassUnion("replValue",   members = c("numeric", "logical", "complex", "raw"))
+setClassUnion("replValueSp", members = c("replValue", "sparseVector", "Matrix"))
+
 
 setClass("determinant",
-	 representation(modulus = "numeric",
-			logarithm = "logical",
-			sign = "integer",
-			call = "call"))
+	 slots = c(modulus = "numeric",
+		   logarithm = "logical",
+		   sign = "integer",
+		   call = "call"))
