@@ -134,33 +134,33 @@ intI <- function(i, n, dn, give.dn = TRUE)
 
     has.dn <- !is.null.DN(dn)
     DN <- has.dn && give.dn
-    if(is(i, "numeric")) {
+    if(is.numeric(i) || is(i, "numeric")) { # inherits(<integer>, "numeric") is FALSE
 	storage.mode(i) <- "integer"
-	if(any(is.na(i)))
-	    stop("'NA' indices are not (yet?) supported for sparse Matrices")
+	if(anyNA(i)) stop("'NA' indices are not (yet?) supported for sparse Matrices")
 	if(any(i < 0L)) {
 	    if(any(i > 0L))
 		stop("you cannot mix negative and positive indices")
 	    i0 <- (0:(n - 1L))[i]
 	} else {
-	    if(length(i) && max(i, na.rm=TRUE) > n)
+	    if(length(i) && max(i, na.rm=TRUE) > n) # base has "subscript out of bounds":
 		stop(gettextf("index larger than maximal %d", n), domain=NA)
 	    if(any(z <- i == 0)) i <- i[!z]
 	    i0 <- i - 1L		# transform to 0-indexing
 	}
 	if(DN) dn <- dn[i]
     }
-    else if (is(i, "logical")) {
+    else if (is.logical(i) || inherits(i, "logical")) {
 	if(length(i) > n)
 	    stop(gettextf("logical subscript too long (%d, should be %d)",
 			  length(i), n), domain=NA)
+	if(anyNA(i)) stop("'NA' indices are not (yet?) supported for sparse Matrices")
 	i0 <- (0:(n - 1L))[i]
 	if(DN) dn <- dn[i]
     } else { ## character
 	if(!has.dn)
 	    stop("no 'dimnames[[.]]': cannot use character indexing")
 	i0 <- match(i, dn)
-	if(any(is.na(i0))) stop("invalid character indexing")
+	if(anyNA(i0)) stop("invalid character indexing")
 	if(DN) dn <- dn[i0]
 	i0 <- i0 - 1L
     }
@@ -189,7 +189,7 @@ intI <- function(i, n, dn, give.dn = TRUE)
 	   i0 = i0, anyDup = anyDup, dn = intIlist$dn),
       ## actually,  iDup  is rarely needed in calling code
       if(anyDup) list(iDup = iDup, i0i = i0i, i.x = i.x,
-		      jm = unlist(jm), i.xtra = rep.int(i.x, sapply(jm, length))))
+		      jm = unlist(jm), i.xtra = rep.int(i.x, lengths(jm))))
 } ## {.ind.prep}
 
 ##' <description>
@@ -310,18 +310,18 @@ replTmat <- function (x, i, j, ..., value)
     ## "FIXME": could pass this (and much ? more) when this function would not *be* a
     ## method but be *called* from methods
 
-    clDv <- getClassDef(clV <- class(value))
+    clDv <- getClassDef(class(value))
     spV <- extends(clDv, "sparseVector")
     ## own version of all0() that works both for sparseVector and atomic vectors:
     .all0 <- function(v) if(spV) length(v@i) == 0 else all0(v)
     delayedAssign("value.not.logical",
                   !(if(spV) {
-                      extends(clDv, "lsparseVector") || extends(clDv, "nsparseVector")
+                      extends1of(clDv, "lsparseVector", "nsparseVector")
                   } else {
                       is.logical(value) || is.logical(as.vector(value))
                   }))
     na <- nargs()
-    if(na == 3) { ## i = vector (but *not* 2-col) indexing"  M[i] <- v
+    if(na == 3) { ## i = vector indexing  M[i] <- v,  e.g.,  M[TRUE] <- v or M[] <- v !
 	Matrix.msg("diagnosing replTmat(x,i,j,v): nargs()= 3; ",
 		   if(iMi | jMi) sprintf("missing (i,j) = (%d,%d)", iMi,jMi))
 	if(iMi) stop("internal bug: missing 'i' in replTmat(): please report")
@@ -339,17 +339,21 @@ replTmat <- function (x, i, j, ..., value)
 		       cl," to ",class(x))
 	}
 	nr <- di[1]
-	x.i <- .Call(m_encodeInd2, x@i, x@j, di=di, FALSE)
-	if(anyDuplicated(x.i)) { ## == if(is_duplicatedT(x, di = di))
+	x.i <- .Call(m_encodeInd2, x@i, x@j, di=di, FALSE, FALSE)
+	if(anyDuplicated(x.i)) { ## == if(anyDuplicatedT(x, di = di))
 	    x <- uniqTsparse(x)
-	    x.i <- .Call(m_encodeInd2, x@i, x@j, di=di, FALSE)
+	    x.i <- .Call(m_encodeInd2, x@i, x@j, di=di, FALSE, FALSE)
 	}
 
         n <- prod(di)
 	i <- if(is.logical(i)) { # full-size logical indexing
 	    if(n) {
-		if(length(i) < n) i <- rep_len(i, n)
-		(0:(n-1))[i] # -> 0-based index vector as well {maybe LARGE!}
+                if(isTRUE(i)) # shortcut
+                    0:(n-1)
+                else {
+                    if(length(i) < n) i <- rep_len(i, n)
+                    (0:(n-1))[i] # -> 0-based index vector as well {maybe LARGE!}
+                }
 	    } else integer(0)
 	} else {
 	    ## also works with *negative* indices etc:
@@ -382,7 +386,7 @@ replTmat <- function (x, i, j, ..., value)
 		x@j <- x@j[sel]
 		if(has.x)
 		    x@x <- x@x[sel]
-		if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
+		if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
 		    x@factors <- list()
 	    }
 	    return(x)
@@ -413,7 +417,7 @@ replTmat <- function (x, i, j, ..., value)
 
 	## 1) Change the matching non-zero entries
 	if(has.x)
-	    x@x[mi[isE]] <- value[isE]
+	    x@x[mi[isE]] <- as(value[isE], class(x@x))
         else if(any0(value[isE])) { ## "n.TMatrix" : remove (i,j) where value is FALSE
             get0 <- !value[isE] ## x[i,j] is TRUE, should become FALSE
             i.rm <- - mi[isE][get0]
@@ -423,14 +427,19 @@ replTmat <- function (x, i, j, ..., value)
 	## 2) add the new non-zero entries
 	i <- i[!isE]
 	xv <- value[!isE]
-	if(has.x) {
-            x@x <- c(x@x, xv)
-	} else { # n.TMatrix : assign (i,j) only where value is TRUE:
-	    i <- i[xv]
+	## --- Be be efficient when  'value' is sparse :
+	if(length(notE <- which(isN0(xv)))) { # isN0(): non-0's; NAs counted too
+	    xv <- xv[notE]
+	    i <- i[notE]
+	    if(has.x) {
+		x@x <- c(x@x, as(xv, class(x@x)))
+	    } else { # n.TMatrix : assign (i,j) only where value is TRUE:
+		i <- i[xv]
+	    }
+	    x@i <- c(x@i, i %%  nr)
+	    x@j <- c(x@j, i %/% nr)
 	}
-	x@i <- c(x@i, i %%  nr)
-	x@j <- c(x@j, i %/% nr)
-	if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
+	if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
 	    x@factors <- list()
 	return(x)
     } ## {nargs = 3;  x[ii] <- value }
@@ -487,7 +496,7 @@ replTmat <- function (x, i, j, ..., value)
     clDx <- getClassDef(clx) # extends() , is() etc all use the class definition
     stopifnot(extends(clDx, "TsparseMatrix"))
     ## Tmatrix maybe non-unique, have an entry split into a sum of several ones:
-    if(is_duplicatedT(x, di = di))
+    if(anyDuplicatedT(x, di = di))
 	x <- uniqTsparse(x)
 
     toGeneral <- r.sym <- FALSE
@@ -525,7 +534,7 @@ replTmat <- function (x, i, j, ..., value)
 
     ## TODO (efficiency): replace  'sel' by 'which(sel)'
     get.ind.sel <- function(ii,ij)
-	(match(x@i, ii, nomatch = 0) & match(x@j, ij, nomatch = 0))
+	(match(x@i, ii, nomatch = 0L) & match(x@j, ij, nomatch = 0L))
     ## sel[k] := TRUE iff k-th non-zero entry (typically x@x[k]) is to be replaced
     sel <- get.ind.sel(i1,i2)
 
@@ -538,7 +547,7 @@ replTmat <- function (x, i, j, ..., value)
 	    x@j <- x@j[!sel]
             if(has.x)
 		x@x <- x@x[!sel]
-	    if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
+	    if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
 		x@factors <- list()
 	}
 	return(x)
@@ -549,7 +558,7 @@ replTmat <- function (x, i, j, ..., value)
     ## now have  lenV <= lenRepl
 
     if(!has.x && # <==> "n.TMatrix"
-       ((iNA <- any(is.na(value))) || value.not.logical))
+       ((iNA <- anyNA(value)) || value.not.logical))
 	warning(if(iNA)
 		gettextf("x[.,.] <- val: x is %s, val not in {TRUE, FALSE} is coerced NA |--> TRUE.",
 			 dQuote(clx))
@@ -569,7 +578,7 @@ replTmat <- function (x, i, j, ..., value)
             if(has.x)
                 x@x <- c(x@x, value)
         }
-	if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
+	if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
 	    x@factors <- list()
         return(x)
     }
@@ -580,7 +589,7 @@ replTmat <- function (x, i, j, ..., value)
 ## FIXME: just for testing !!
 ## if(identical(Sys.getenv("USER"),"maechler")
 ##    if(lenRepl > 2) { # __________ ___ JUST for testing! _______________
-	if(is.null(v <- getOption("Matrix.quiet")) || !v)
+	if(nonTRUEoption("Matrix.quiet"))
 	    message(gettextf("x[.,.] <- val : x being coerced from Tsparse* to CsparseMatrix"),
 		    domain = NA)
 	return(replCmat4(as(x,"CsparseMatrix"), i1, i2, iMi=iMi, jMi=jMi,
@@ -616,9 +625,10 @@ replTmat <- function (x, i, j, ..., value)
 
     if(any(sel)) {
 	## the 0-based indices of non-zero entries -- WRT to submatrix
-	non0 <- cbind(match(x@i[sel], i1),
-		      match(x@j[sel], i2)) - 1L
-	iN0 <- 1L + .Call(m_encodeInd, non0, di = dind, FALSE)
+	iN0 <- 1L + .Call(m_encodeInd2,
+			  match(x@i[sel], i1),
+			  match(x@j[sel], i2),
+			  di = dind, orig1=TRUE, FALSE)
 
 	## 1a) replace those that are already non-zero with non-0 values
 	vN0 <- isN0(value[iN0])
@@ -628,8 +638,8 @@ replTmat <- function (x, i, j, ..., value)
 	}
 
 	## 1b) replace non-zeros with 0 --> drop entries
-	if(any(!vN0)) {
-	    ii <- which(sel)[!vN0]
+	if(!all(vN0)) { ##-> ii will not be empty
+	    ii <- which(sel)[which(!vN0)] # <- vN0 may be sparseVector
 	    if(has.x)
 		x@x <- x@x[-ii]
 	    x@i <- x@i[-ii]
@@ -665,7 +675,7 @@ replTmat <- function (x, i, j, ..., value)
 		x@x <- c(x@x, as.vector(value[iIN0]))
 	}
     }
-    if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
+    if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
 	x@factors <- list()
     x
 } ## end{replTmat}
@@ -685,7 +695,7 @@ replTmat <- function (x, i, j, ..., value)
 	## c(i) : drop "matrix" to logical vector
 	x[as.vector(i)] <- value
 	return(x)
-    } else if(extends(cli <- getClassDef(class(i)),"lMatrix") || extends(cli, "nMatrix")) {
+    } else if(extends1of(cli <- getClassDef(class(i)), c("lMatrix", "nMatrix"))) {
 	Matrix.msg(".TM.repl.i.mat(): \"lMatrix\" case ...", .M.level=2)
 	i <- which(as(i, if(extends(cli, "sparseMatrix")) "sparseVector" else "vector"))
 	## x[i] <- value ; return(x)
@@ -699,7 +709,7 @@ replTmat <- function (x, i, j, ..., value)
     if(!is.integer(i)) storage.mode(i) <- "integer"
     if(any(i < 0))
 	stop("negative values are not allowed in a matrix subscript")
-    if(any(is.na(i)))
+    if(anyNA(i))
 	stop("NAs are not allowed in subscripted assignments")
     if(any(i0 <- (i == 0))) # remove them
 	i <- i[ - which(i0, arr.ind = TRUE)[,"row"], ]
@@ -730,7 +740,7 @@ replTmat <- function (x, i, j, ..., value)
     if(any(i2 > nc)) stop(gettextf("column indices must be <= ncol(.) which is %d", nc), domain=NA)
 
     ## Tmatrix maybe non-unique, have an entry split into a sum of several ones:
-    if(is_duplicatedT(x, di = di))
+    if(anyDuplicatedT(x, di = di))
 	x <- uniqTsparse(x)
 
     toGeneral <- FALSE
@@ -782,7 +792,7 @@ replTmat <- function (x, i, j, ..., value)
 	clDx <- getClassDef(clx <- class(x))
     }
 
-    ii.v <- .Call(m_encodeInd, i - 1L, di, checkBounds = TRUE)# 0-indexing
+    ii.v <- .Call(m_encodeInd, i, di, orig1=TRUE, checkBounds = TRUE)
     if(id <- anyDuplicated(ii.v, fromLast=TRUE)) {
         Matrix.msg("duplicate ij-entries in 'Matrix[ ij ] <- value'; using last",
                    .M.level = 1)
@@ -794,7 +804,7 @@ replTmat <- function (x, i, j, ..., value)
             value <- value[nd]
         }
     }
-    ii.x <- .Call(m_encodeInd2, x@i, x@j, di, FALSE)
+    ii.x <- .Call(m_encodeInd2, x@i, x@j, di, FALSE, FALSE)
     m1 <- match(ii.v, ii.x)
     i.repl <- !is.na(m1) # those that need to be *replaced*
 
@@ -825,7 +835,7 @@ replTmat <- function (x, i, j, ..., value)
 	    x@x <- c(x@x, value[i.new])
     }
 
-    if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
+    if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
 	x@factors <- list()
     x
 } ## end{.TM.repl.i.mat}
@@ -898,3 +908,12 @@ setMethod("t", signature(x = "TsparseMatrix"),
 	      r@Dimnames <- x@Dimnames[2:1]
 	      r
       })
+
+isDiagTsp <- function(object) {
+    d <- dim(object)
+    if(d[1] != d[2])
+        FALSE
+    else
+        length(i <- object@i) == length(j <- object@j) && all(i == j)
+}
+setMethod("isDiagonal", signature(object = "TsparseMatrix"), isDiagTsp)
