@@ -39,7 +39,10 @@ Diagonal <- function(n, x = NULL)
 			    unitri, kind,
 			    cols = if(n) 0:(n - 1L) else integer(0))
 {
-    stopifnot(n == (n. <- as.integer(n)), (n <- n.) >= 0)
+    n <- if (missing(n)) length(x) else {
+	stopifnot(length(n) == 1, n == as.integer(n), n >= 0)
+	as.integer(n)
+    }
     if(!(mcols <- missing(cols)))
 	stopifnot(0 <= (cols <- as.integer(cols)), cols < n)
     m <- length(cols)
@@ -79,12 +82,13 @@ Diagonal <- function(n, x = NULL)
 }
 
 ## Pkg 'spdep' had (relatively slow) versions of this as_dsCMatrix_I()
-.symDiagonal <- function(n, x = rep.int(1,n), uplo = "U")
-    .sparseDiagonal(n, x, uplo, shape = "s")
+.symDiagonal <- function(n, x = rep.int(1,n), uplo = "U", kind)
+    .sparseDiagonal(n, x, uplo, shape = "s", kind = kind)
 
+## NOTA BENE: .triDiagonal() would be misleading (<=> banded tri-diagonal matrix !)
 # instead of   diagU2N(as(Diagonal(n), "CsparseMatrix")), diag = "N" in any case:
-.trDiagonal <- function(n, x = 1, uplo = "U", unitri=TRUE)
-    .sparseDiagonal(n, x, uplo, shape = "t", unitri=unitri)
+.trDiagonal <- function(n, x = 1, uplo = "U", unitri = TRUE, kind)
+    .sparseDiagonal(n, x, uplo, shape = "t", unitri=unitri, kind=kind)
 
 
 ## This is modified from a post of Bert Gunter to R-help on  1 Sep 2005.
@@ -120,6 +124,7 @@ if(FALSE)##--- no longer used:
     ## block-diagonal matrix [a dgTMatrix] from list of matrices
     stopifnot(is.list(lst), (nl <- length(lst)) >= 1)
 
+### FIXME: next line is *slow* when lst = list of 75'000  dense 3x3 matrices
     Tlst <- lapply(lapply(lst, as_Csp2), # includes "diagU2N"
 		   as, "TsparseMatrix")
     if(nl == 1) return(Tlst[[1]])
@@ -128,6 +133,8 @@ if(FALSE)##--- no longer used:
     j_off <- c(0L, cumsum(vapply(Tlst, ncol, 1L)))
 
     clss <- vapply(Tlst, class, "")
+    ## NB ("FIXME"): this requires the component classes to be *called*
+    ## -- "dgTMatrix" | "dnTMatrix" etc (and not just *extend* those)!
     typ <- substr(clss, 2, 2)
     knd <- substr(clss, 1, 1)
     sym <- typ == "s" # symmetric ones
@@ -206,13 +213,22 @@ setMethod("triu", "diagonalMatrix", function(x, k = 0, ...)
 
 
 
-.diag2tT <- function(from, uplo = "U", kind = .M.kind(from)) {
+.diag2tT <- function(from, uplo = "U", kind = .M.kind(from), drop0 = TRUE) {
     ## to triangular Tsparse
-    i <- if(from@diag == "U") integer(0) else seq_len(from@Dim[1]) - 1L
+    x <- from@x
+    i <- if(from@diag == "U")
+	      integer(0L)
+	  else if(drop0 & any0(x)) {
+	      ii <- which(isN0(x))
+	      x <- x[ii]
+	      ii - 1L
+	  }
+	  else
+	      seq_len(from@Dim[1]) - 1L
     new(paste0(kind, "tTMatrix"),
 	diag = from@diag, Dim = from@Dim, Dimnames = from@Dimnames,
 	uplo = uplo,
-	x = from@x, # <- ok for diag = "U" and "N" (!)
+	x = x, # <- ok for diag = "U" and "N" (!)
 	i = i, j = i)
 }
 
@@ -235,8 +251,8 @@ setMethod("triu", "diagonalMatrix", function(x, k = 0, ...)
 }
 
 ## diagonal -> triangular,  upper / lower depending on "partner" 'x':
-diag2tT.u <- function(d, x, kind = .M.kind(d))
-    .diag2tT(d, uplo = if(is(x,"triangularMatrix")) x@uplo else "U", kind)
+diag2tT.u <- function(d, x, kind = .M.kind(d), drop0 = TRUE)
+    .diag2tT(d, uplo = if(is(x,"triangularMatrix")) x@uplo else "U", kind, drop0)
 
 ## diagonal -> sparse {triangular OR symmetric} (upper / lower) depending on "partner":
 diag2Tsmart <- function(d, x, kind = .M.kind(d)) {
@@ -265,7 +281,11 @@ setAs("ddiMatrix", "triangularMatrix", di2tT)
 setAs("ddiMatrix", "TsparseMatrix", di2tT)
 setAs("ddiMatrix", "dsparseMatrix", di2tT)
 setAs("ddiMatrix", "CsparseMatrix",
-      function(from) as(.diag2tT(from, "U", "d"), "CsparseMatrix"))
+      function(from) .T2Cmat(.diag2tT(from, "U", "d"), isTri=TRUE))
+## Such that  as(Matrix(0, d,d), "dgCMatrix")  continues working:
+setAs("ddiMatrix", "dgCMatrix",
+      function(from) .T2Cmat(.diag2tT(from, "U", "d"), isTri=TRUE))
+
 setAs("ddiMatrix", "symmetricMatrix", function(from) .diag2sT(from, "U", "d"))
 ##
 ## ldi*:
@@ -276,7 +296,7 @@ setAs("ldiMatrix", "triangularMatrix", di2tT)
 setAs("ldiMatrix", "TsparseMatrix", di2tT)
 setAs("ldiMatrix", "lsparseMatrix", di2tT)
 setAs("ldiMatrix", "CsparseMatrix",
-      function(from) as(.diag2tT(from, "U", "l"), "CsparseMatrix"))
+      function(from) .T2Cmat(.diag2tT(from, "U", "l"), isTri=TRUE))
 setAs("ldiMatrix", "symmetricMatrix", function(from) .diag2sT(from, "U", "l"))
 rm(di2tT)
 
@@ -348,7 +368,7 @@ setAs("matrix", "diagonalMatrix",
 	  if(d[1] != (n <- d[2])) stop("non-square matrix")
 	  if(any(from[row(from) != col(from)] != 0))
 	      stop("matrix with non-zero off-diagonals cannot be coerced to \"diagonalMatrix\"")
-	  x <- diag(from)
+	  x <- diag(from); names(x) <- NULL # don't want them in 'x' slot
 	  if(is.logical(x)) {
 	      cl <- "ldiMatrix"
 	      uni <- allTrue(x) ## uni := {is it unit-diagonal ?}
@@ -368,7 +388,7 @@ setAs("Matrix", "diagonalMatrix",
 	  if(d[1] != (n <- d[2])) stop("non-square matrix")
 	  if(!isDiagonal(from)) stop("matrix is not diagonal")
 	  ## else:
-	  x <- diag(from)
+	  x <- diag(from); names(x) <- NULL # don't want them in 'x' slot
 	  if(is.logical(x)) {
 	      cl <- "ldiMatrix"
 	      uni <- allTrue(x)
@@ -413,9 +433,8 @@ setMethod("[", signature(x = "diagonalMatrix", i = "missing",
 
 ## When you assign to a diagonalMatrix, the result should be
 ## diagonal or sparse ---
-## FIXME: this now fails because the "denseMatrix" methods come first in dispatch
-## Only(?) current bug:  x[i] <- value  is wrong when  i is *vector*
 replDiag <- function(x, i, j, ..., value) {
+## FIXME: if   (i == j)  &&  isSymmetric(value) then -- want symmetricMatrix result! -- or diagMatrix
     x <- as(x, "CsparseMatrix")# was "Tsparse.." till 2012-07
     if(missing(i))
 	x[, j] <- value
@@ -430,7 +449,11 @@ replDiag <- function(x, i, j, ..., value) {
 			   na), domain=NA)
     } else
 	x[i,j] <- value
-    if(isDiagonal(x)) as(x, "diagonalMatrix") else x
+    ## TODO: the following is a bit expensive; have cases above e.g. [i,] where
+    ## ----- we could check *much* faster :
+    if(isDiagonal(x))   as(x, "diagonalMatrix") else
+    if(isTriangular(x)) as(x, "triangularMatrix") else
+    if(isSymmetric(x))  as(x, "symmetricMatrix") else x
 }
 
 setReplaceMethod("[", signature(x = "diagonalMatrix", i = "index",
@@ -483,7 +506,11 @@ setReplaceMethod("[", signature(x = "diagonalMatrix",
 			     x@x[ii] <- value
 			     x
 			 } else { ## no longer diagonal, but remain sparse:
-			     x <- as(x, "TsparseMatrix")
+###  FIXME:  use  uplo="U" or uplo="L"  (or *not* "triangularMatrix") depending on LE <- i <= j
+###          all(LE) //  all(!LE) // remaining cases
+
+## --> use                        .diag2tT(from, uplo = "U", kind = .M.kind(from))
+			     x <- as(x, "triangularMatrix") # was "TsparseMatrix"
 			     x[i] <- value
 			     x
 			 }
@@ -532,15 +559,16 @@ setMethod("isSymmetric",  "diagonalMatrix", function(object, ...) TRUE)
 setMethod("symmpart", signature(x = "diagonalMatrix"), function(x) x)
 setMethod("skewpart", signature(x = "diagonalMatrix"), function(x) .setZero(x))
 
-setMethod("chol", signature(x = "ddiMatrix"),
-	  function(x, pivot, ...) {
-	      if(x@diag == "U") return(x)
-	      ## else
-	      if(any(x@x < 0))
-		  stop("chol() is undefined for diagonal matrix with negative entries")
-	      x@x <- sqrt(x@x)
-	      x
-	  })
+cholDiag <- function(x, pivot, ...) { ## x : typically "ddiMatrix"
+    if(x@diag == "U") return(x)
+    ## else
+    if(any(x@x < 0))
+        stop("chol() is undefined for diagonal matrix with negative entries")
+    x@x <- sqrt(x@x)
+    x
+}
+setMethod("chol", signature(x = "ddiMatrix"), cholDiag)
+
 ## chol(L) is L for logical diagonal:
 setMethod("chol", signature(x = "ldiMatrix"), function(x, pivot, ...) x)
 
@@ -553,7 +581,7 @@ setMethod("norm", signature(x = "diagonalMatrix", type = "character"),
 	      type <- toupper(substr(type[1], 1, 1))
 	      isU <- (x@diag == "U") # unit-diagonal
 	      if(type == "F") sqrt(if(isU) n else sum(x@x^2))
-	      else { ## norm == "I","1","O","M" :
+	      else { ## norm == "I","1","O","M","2" :
 		  if(isU) 1 else max(abs(x@x))
 	      }
 	  })
@@ -653,30 +681,40 @@ setMethod("tcrossprod", signature(x = "diagonalMatrix", y = "missing"), diagprod
 ## analogous to matdiagprod() below:
 diagmatprod <- function(x, y) {
     ## x is diagonalMatrix
-    if(x@Dim[2] != nrow(y)) stop("non-matching dimensions")
-    Matrix(if(x@diag == "U") y else x@x * y)
+    dy <- dim(y)
+    if(x@Dim[2L] != dy[1L]) stop("non-matching dimensions")
+    if(prod(dy))
+	Matrix(if(x@diag == "U") y else x@x * y)
+    else
+	Matrix(if(x@diag == "U") y else x@x * y, nrow=dy[1L], ncol=dy[2L])
 }
 setMethod("%*%", signature(x = "diagonalMatrix", y = "matrix"), diagmatprod)
 
 ##formals(diagmatprod) <- alist(x=, y=NULL, boolArith = NA, ...=) ## FIXME boolArith
 diagmatprod2 <- function(x, y=NULL, boolArith = NA, ...) {
     ## x is diagonalMatrix
-    if(x@Dim[2] != nrow(y)) stop("non-matching dimensions")
-    Matrix(if(x@diag == "U") y else x@x * y)
+    dy <- dim(y)
+    if(x@Dim[2L] != dy[1L]) stop("non-matching dimensions")
+    if(prod(dy))
+	Matrix(if(x@diag == "U") y else x@x * y)
+    else
+	Matrix(if(x@diag == "U") y else x@x * y, nrow=dy[1L], ncol=dy[2L])
 }
 setMethod("crossprod",  signature(x = "diagonalMatrix", y = "matrix"), diagmatprod2)
 
 diagGeprod <- function(x, y) {
-    if(x@Dim[2] != y@Dim[1]) stop("non-matching dimensions")
-    if(x@diag != "U")
+    if(x@Dim[2L] != y@Dim[1L]) stop("non-matching dimensions")
+    if(x@diag != "U") {
+        if(!is.numeric(y@x)) y <- as(y, "dMatrix")
         y@x <- x@x * y@x
+    }
     y
 }
 setMethod("%*%", signature(x= "diagonalMatrix", y= "dgeMatrix"), diagGeprod)
 setMethod("%*%", signature(x= "diagonalMatrix", y= "lgeMatrix"), diagGeprod)
 
 diagGeprodBool <- function(x, y) {
-    if(x@Dim[2] != y@Dim[1]) stop("non-matching dimensions")
+    if(x@Dim[2L] != y@Dim[1L]) stop("non-matching dimensions")
     if(!is.logical(y@x)) y <- as(y, "lMatrix")
     if(x@diag != "U")
         y@x <- x@x & y@x
@@ -685,9 +723,10 @@ diagGeprodBool <- function(x, y) {
 setMethod("%&%", signature(x= "diagonalMatrix", y= "geMatrix"), diagGeprodBool)
 
 diagGeprod2 <- function(x, y=NULL, boolArith = NA, ...) {
-    if(x@Dim[2] != y@Dim[1]) stop("non-matching dimensions")
+    if(x@Dim[2L] != y@Dim[1L]) stop("non-matching dimensions")
     bool <- isTRUE(boolArith)
     if(bool && !is.logical(y@x)) y <- as(y, "lMatrix")
+    else if(!bool && !is.numeric(y@x)) y <- as(y, "dMatrix")
     if(x@diag != "U")
         y@x <- if(bool) x@x & y@x else x@x * y@x
     y
@@ -699,16 +738,18 @@ setMethod("crossprod", signature(x = "diagonalMatrix", y = "lgeMatrix"), diagGep
 ## analogous to diagmatprod() above:
 matdiagprod <- function(x, y) {
     dx <- dim(x)
-    if(dx[2] != y@Dim[1]) stop("non-matching dimensions")
-    Matrix(if(y@diag == "U") x else x * rep(y@x, each = dx[1]))
+    if(dx[2L] != y@Dim[1L]) stop("non-matching dimensions")
+    Matrix(if(y@diag == "U") x else x * rep(y@x, each = dx[1L]))
 }
 setMethod("%*%", signature(x = "matrix", y = "diagonalMatrix"), matdiagprod)
 
 gediagprod <- function(x, y) {
     dx <- dim(x)
-    if(dx[2] != y@Dim[1]) stop("non-matching dimensions")
-    if(y@diag == "N")
-	x@x <- x@x * rep(y@x, each = dx[1])
+    if(dx[2L] != y@Dim[1L]) stop("non-matching dimensions")
+    if(y@diag == "N") {
+        if(!is.numeric(x@x)) x <- as(x, "dMatrix")
+	x@x <- x@x * rep(y@x, each = dx[1L])
+    }
     x
 }
 setMethod("%*%", signature(x= "dgeMatrix", y= "diagonalMatrix"), gediagprod)
@@ -716,10 +757,10 @@ setMethod("%*%", signature(x= "lgeMatrix", y= "diagonalMatrix"), gediagprod)
 
 gediagprodBool <- function(x, y) {
     dx <- dim(x)
-    if(dx[2] != y@Dim[1]) stop("non-matching dimensions")
+    if(dx[2L] != y@Dim[1L]) stop("non-matching dimensions")
     if(!is.logical(x@x)) x <- as(x, "lMatrix")
     if(y@diag == "N")
-	x@x <- x@x & rep(y@x, each = dx[1])
+	x@x <- x@x & rep(y@x, each = dx[1L])
     x
 }
 setMethod("%&%", signature(x= "geMatrix", y= "diagonalMatrix"), gediagprodBool)
@@ -727,34 +768,35 @@ setMethod("%&%", signature(x= "geMatrix", y= "diagonalMatrix"), gediagprodBool)
 setMethod("tcrossprod",signature(x = "matrix", y = "diagonalMatrix"),
           function(x, y=NULL, boolArith = NA, ...) {
               dx <- dim(x)
-              if(dx[2] != y@Dim[1]) stop("non-matching dimensions")
+              if(dx[2L] != y@Dim[1L]) stop("non-matching dimensions")
               bool <- isTRUE(boolArith)
               if(bool && !is.logical(y@x)) y <- as(y, "lMatrix")
               Matrix(if(y@diag == "U") x else
-                     if(bool) x & rep(y@x, each = dx[1])
-                     else     x * rep(y@x, each = dx[1]))
+                     if(bool) x & rep(y@x, each = dx[1L])
+                     else     x * rep(y@x, each = dx[1L]))
           })
 
 setMethod("crossprod", signature(x = "matrix", y = "diagonalMatrix"),
 	  function(x, y=NULL, boolArith = NA, ...) {
 	      dx <- dim(x)
-	      if(dx[1] != y@Dim[1]) stop("non-matching dimensions")
+	      if(dx[1L] != y@Dim[1L]) stop("non-matching dimensions")
               bool <- isTRUE(boolArith)
               if(bool && !is.logical(y@x)) y <- as(y, "lMatrix")
 	      Matrix(if(y@diag == "U") t(x) else
-		     if(bool) t(rep.int(y@x, dx[2]) & x)
-		     else     t(rep.int(y@x, dx[2]) * x))
+		     if(bool) t(rep.int(y@x, dx[2L]) & x)
+		     else     t(rep.int(y@x, dx[2L]) * x))
 	  })
 
 
 gediagprod2 <- function(x, y=NULL, boolArith = NA, ...) {
     dx <- dim(x)
-    if(dx[2] != y@Dim[1]) stop("non-matching dimensions")
+    if(dx[2L] != y@Dim[1L]) stop("non-matching dimensions")
     bool <- isTRUE(boolArith)
     if(bool && !is.logical(x@x)) x <- as(x, "lMatrix")
+    else if(!bool && !is.numeric(x@x)) x <- as(x, "dMatrix")
     if(y@diag == "N")
-	x@x <- if(bool) x@x & rep(y@x, each = dx[1])
-	       else     x@x * rep(y@x, each = dx[1])
+	x@x <- if(bool) x@x & rep(y@x, each = dx[1L])
+	       else     x@x * rep(y@x, each = dx[1L])
     x
 }
 setMethod("tcrossprod", signature(x = "dgeMatrix", y = "diagonalMatrix"), gediagprod2)
@@ -780,7 +822,7 @@ setMethod("%*%", signature(x = "denseMatrix", y = "diagonalMatrix"),
 ##' @param y diagonalMatrix
 ##' @return x %*% y
 Cspdiagprod <- function(x, y, boolArith = NA, ...) {
-    if((m <- ncol(x)) != y@Dim[1]) stop("non-matching dimensions")
+    if((m <- ncol(x)) != y@Dim[1L]) stop("non-matching dimensions")
     if(y@diag == "N") { ## otherwise: y == Diagonal(n) : multiplication is identity
 	x <- .Call(Csparse_diagU2N, x)
 	cx <- getClass(class(x))
@@ -811,7 +853,7 @@ Cspdiagprod <- function(x, y, boolArith = NA, ...) {
 	    as(x, "lMatrix")
 	} else {
 	    ## else boolArith is  NA or FALSE {which are equivalent here, das diagonal = "numLike"}
-	    if(extends(cx, "nMatrix") || extends(cx, "lMatrix"))
+	    if(extends1of(cx, c("nMatrix", "lMatrix")))
 		as(x, "dMatrix") else x
 	}
     }
@@ -821,7 +863,7 @@ Cspdiagprod <- function(x, y, boolArith = NA, ...) {
 ##' @param y CsparseMatrix
 ##' @return x %*% y
 diagCspprod <- function(x, y, boolArith = NA, ...) {
-    if(x@Dim[2] != y@Dim[1]) stop("non-matching dimensions")
+    if(x@Dim[2L] != y@Dim[1L]) stop("non-matching dimensions")
     if(x@diag == "N") {
 	y <- .Call(Csparse_diagU2N, y)
 	cy <- getClass(class(y))
@@ -856,7 +898,7 @@ diagCspprod <- function(x, y, boolArith = NA, ...) {
 	    as(y, "lMatrix")
 	} else {
 	    ## else boolArith is  NA or FALSE {which are equivalent here, das diagonal = "numLike"}
-	    if(extends(cy, "nMatrix") || extends(cy, "lMatrix"))
+	    if(extends1of(cy, c("nMatrix", "lMatrix")))
 		as(y, "dMatrix") else y
 	}
     }
@@ -928,7 +970,7 @@ setMethod("solve", signature(a = "diagonalMatrix", b = "missing"),
 	  })
 
 solveDiag <- function(a, b, ...) {
-    if(a@Dim[1] != nrow(b))
+    if(a@Dim[1L] != nrow(b))
         stop("incompatible matrix dimensions")
     ## trivially invert a 'in place' and multiply:
     a@x <- 1/ a@x
@@ -975,7 +1017,7 @@ diagOdiag <- function(e1,e2) {
 	isLog <- (is.logical(r) || is.logical(r00))
         Matrix.msg("exploding <diag> o <diag> into dense matrix", .M.level = 2)
 	d <- e1@Dim
-	n <- d[1]
+	n <- d[1L]
 	stopifnot(length(r) == n)
 	if(isNum && !is.double(r)) r <- as.double(r)
 	xx <- as.vector(matrix(rbind(r, matrix(r00,n,n)), n,n))
@@ -1009,7 +1051,7 @@ diagOtri <- function(e1,e2) {
     r <- callGeneric(d1 <- .diag.x(e1), diag(e2)) # error if not "compatible"
     ## Check what happens with non-diagonals, i.e. (0 o 0), (FALSE o 0), ...:
     e1.0 <- if(is.numeric(d1)) 0 else FALSE
-    r00 <- callGeneric(e1.0, if(.n2 <- is.numeric(e2[0])) 0 else FALSE)
+    r00 <- callGeneric(e1.0, if(.n2 <- is.numeric(e2[0L])) 0 else FALSE)
     if(is0(r00)) { ##  r00 == 0 or FALSE --- result *is* triangular
         diag(e2) <- r
         ## check what happens "in the triangle"
@@ -1091,7 +1133,9 @@ setMethod("Ops", signature(e1 = "sparseMatrix", e2 = "ldiMatrix"),
 for(arg2 in c("numeric","logical"))
 setMethod("Arith", signature(e1 = "ddiMatrix", e2 = arg2),
 	  function(e1,e2) {
-	      n <- e1@Dim[1]
+	      n <- e1@Dim[1L]
+	      if(length(e2) == 0L)
+		  return(if(n) numeric() else e1)
 	      f0 <- callGeneric(0, e2)
 	      if(all0(f0)) { # remain diagonal
 		  L1 <- (le <- length(e2)) == 1L
@@ -1113,7 +1157,9 @@ setMethod("Arith", signature(e1 = "ddiMatrix", e2 = arg2),
 for(arg1 in c("numeric","logical"))
 setMethod("Arith", signature(e1 = arg1, e2 = "ddiMatrix"),
 	  function(e1,e2) {
-	      n <- e2@Dim[1]
+	      n <- e2@Dim[1L]
+	      if(length(e1) == 0L)
+		  return(if(n) numeric() else e2)
 	      f0 <- callGeneric(e1, 0)
 	      if(all0(f0)) { # remain diagonal
 		  L1 <- (le <- length(e1)) == 1L
@@ -1136,7 +1182,10 @@ setMethod("Arith", signature(e1 = arg1, e2 = "ddiMatrix"),
 for(arg2 in c("numeric","logical"))
 setMethod("Arith", signature(e1 = "ldiMatrix", e2 = arg2),
 	  function(e1,e2) {
-	      n <- e1@Dim[1]
+	      n <- e1@Dim[1L]
+	      if(length(e2) == 0L)
+		  return(if(n) numeric()
+			 else copyClass(e1, "ddiMatrix", c("diag", "Dim", "Dimnames"), check=FALSE))
 	      f0 <- callGeneric(0, e2)
 	      if(all0(f0)) { # remain diagonal
 		  L1 <- (le <- length(e2)) == 1L
@@ -1160,7 +1209,10 @@ setMethod("Arith", signature(e1 = "ldiMatrix", e2 = arg2),
 for(arg1 in c("numeric","logical"))
 setMethod("Arith", signature(e1 = arg1, e2 = "ldiMatrix"),
 	  function(e1,e2) {
-	      n <- e2@Dim[1]
+	      n <- e2@Dim[1L]
+	      if(length(e1) == 0L)
+		  return(if(n) numeric()
+			 else copyClass(e2, "ddiMatrix", c("diag", "Dim", "Dimnames"), check=FALSE))
 	      f0 <- callGeneric(e1, 0)
 	      if(all0(f0)) { # remain diagonal
 		  L1 <- (le <- length(e1)) == 1L
@@ -1192,7 +1244,10 @@ if(FALSE) {
 for(arg2 in c("numeric","logical"))
 setMethod("Ops", signature(e1 = "ddiMatrix", e2 = arg2),
 	  function(e1,e2) {
-	      n <- e1@Dim[1]
+	      n <- e1@Dim[1L]
+	      if(length(e2) == 0L)
+		  return(if(n) logical()
+			 else copyClass(e1, "ldiMatrix", c("diag", "Dim", "Dimnames"), check=FALSE))
 	      f0 <- callGeneric(0, e2)
 	      if(all0(f0)) { # remain diagonal
 		  L1 <- (le <- length(e2)) == 1L
@@ -1217,7 +1272,9 @@ setMethod("Ops", signature(e1 = "ddiMatrix", e2 = arg2),
 for(arg2 in c("numeric","logical"))
 setMethod("Ops", signature(e1 = "ldiMatrix", e2 = arg2),
 	  function(e1,e2) {
-	      n <- e1@Dim[1]
+	      n <- e1@Dim[1L]
+	      if(length(e2) == 0L)
+                  return(if(n) logical() else e1)
 	      f0 <- callGeneric(FALSE, e2)
 	      if(all0(f0)) { # remain diagonal
 		  L1 <- (le <- length(e2)) == 1L
@@ -1287,13 +1344,13 @@ setMethod("any", cl,
 	      else if(x@diag == "U") TRUE else any(x@x, ..., na.rm = na.rm)
 	  })
 setMethod("all",  cl, function (x, ..., na.rm) {
-    n <- x@Dim[1]
+    n <- x@Dim[1L]
     if(n >= 2) FALSE
     else if(n == 0 || x@diag == "U") TRUE
     else all(x@x, ..., na.rm = na.rm)
 })
 setMethod("prod", cl, function (x, ..., na.rm) {
-    n <- x@Dim[1]
+    n <- x@Dim[1L]
     if(n >= 2) 0
     else if(n == 0 || x@diag == "U") 1
     else ## n == 1, diag = "N" :
@@ -1303,7 +1360,7 @@ setMethod("prod", cl, function (x, ..., na.rm) {
 setMethod("sum", cl,
 	  function(x, ..., na.rm) {
 	      r <- sum(x@x, ..., na.rm = na.rm)# double or integer, correctly
-	      if(x@diag == "U" && !is.na(r)) r + x@Dim[1] else r
+	      if(x@diag == "U" && !is.na(r)) r + x@Dim[1L] else r
 	  })
 }
 
@@ -1345,8 +1402,8 @@ setMethod("show", signature(object = "diagonalMatrix"),
 	      d <- dim(object)
 	      cl <- class(object)
 	      cat(sprintf('%d x %d diagonal matrix of class "%s"',
-			  d[1], d[2], cl))
-	      if(d[1] < 50) {
+			  d[1L], d[2L], cl))
+	      if(d[1L] < 50) {
 		  cat("\n")
 		  prDiag(object)
 	      } else {
@@ -1365,7 +1422,7 @@ setMethod("summary", signature(object = "diagonalMatrix"),
 	      r <- summary(object@x, ...)
 	      attr(r, "header") <-
 		  sprintf('%d x %d diagonal Matrix of class "%s"',
-			  d[1], d[2], class(object))
+			  d[1L], d[2L], class(object))
 	      ## use ole' S3 technology for such a simple case
 	      class(r) <- c("diagSummary", class(r))
 	      r
