@@ -15,29 +15,54 @@ n
 ## this:
 m <- Matrix::Matrix(cbind(1,0,diag(x=2:4)))
 m
+mt <- m + table(gl(3,5), gl(5,3))# failed in Matrix <= 1.2.9
+mt
+stopifnot(is(mt, "sparseMatrix"))
+
 ##--------------------------------------------------------------------
 
 library(Matrix)
 
 source(system.file("test-tools.R", package = "Matrix"))# identical3() etc
+(doExtras <- doExtras && getRversion() >= "3.4") # so have withAutoprint(.)
 
 if(interactive()) {
-    options(error = recover)
-} else options(Matrix.verbose = TRUE)# to show Matrix.msg()s
+    options(error = recover, Matrix.verbose = TRUE, warn = 1)
+} else options(              Matrix.verbose = TRUE, warn = 1)
+                                        #   ^^^^^^ to show Matrix.msg()s
 
 ### Matrix() ''smartness''
-(d4 <- Matrix(diag(4)))
+(d40 <- Matrix( diag(4)))
 (z4 <- Matrix(0*diag(4)))
 (o4 <- Matrix(1+diag(4)))
 (tr <- Matrix(cbind(1,0:1)))
 (M4 <- Matrix(m4 <- cbind(0,rbind(6*diag(3),0))))
 dM4 <- Matrix(M4, sparse = FALSE)
+d4. <- diag(4); dimnames(d4.) <- dns <- rep(list(LETTERS[1:4]), 2)
+d4a <- diag(4); dimnames(d4a) <- dna <- list(LETTERS[1:4], letters[1:4])# "a"symmetric
+m1a <- matrix(0, dimnames=list("A","b"))# "a"symmetric
+d4di<- as(d4., "diagonalMatrix")
+d4da<- as(d4a, "diagonalMatrix")
+d4d <- as(d4., "denseMatrix")
+d4aS <- Matrix(d4a, sparse=TRUE, doDiag=FALSE)
+d1aS <- Matrix(m1a, sparse=TRUE, doDiag=FALSE)
+stopifnot(exprs = {
+    identical(d4di@x, numeric()) # was "named" unnecessarily
+    identical(dimnames(d4 <- Matrix(d4.)), dns)
+    identical4(d40, Matrix(diag(4)), unname(d4), unname(d4da))
+    identical3(d4, as(d4., "Matrix"), as(d4., "diagonalMatrix"))
+    is(d4aS, "dtCMatrix") # not "dsC*", as asymmetric dimnames
+    is(d4d, "denseMatrix")
+    identical(dimnames(d4T <- as(d4., "TsparseMatrix")), dns) # failed till 2019-09-xx
+    identical(d4T, as(d4., "dgTMatrix"))
+})
+
 class(mN <-  Matrix(NA, 3,4)) # NA *is* logical
 validObject(Matrix(NA))
 bd4 <- bdiag(M4,dM4,M4)
-stopifnot(isValid(o4, "dsyMatrix"),
-          isValid(M4, "dtCMatrix"),
-          validObject(dM4), validObject(mN),
+stopifnotValid(o4, "dsyMatrix")
+stopifnotValid(M4, "dtCMatrix")
+stopifnot(validObject(dM4), validObject(mN),
           identical(bdiag(M4), bdiag(dM4)),
           identical(bd4@p, c(0L,0:3,3:6,6:9)),
           identical(bd4@i, c(0:2, 4:6, 8:10)), bd4@x == 6
@@ -48,33 +73,83 @@ assert.EQ.mat(mN, matrix(NA, 3,4))
 assert.EQ.mat(bdiag(diag(4)), diag(4))
 sL <- Matrix(, 3,4, sparse=TRUE)# -> "lgC"
 trS <- Matrix(tr, sparse=TRUE)# failed in 0.9975-11
-stopifnot(isValid(d4, "diagonalMatrix"),   isValid(z4,  "diagonalMatrix"),
-          isValid(tr, "triangularMatrix"), isValid(trS, "triangularMatrix"),
-          all(is.na(sL@x)), ## not yet:  all(is.na(sL)),
+stopifnotValid(d4, "diagonalMatrix");   stopifnotValid(z4,  "diagonalMatrix")
+stopifnotValid(tr, "triangularMatrix"); stopifnotValid(trS, "triangularMatrix")
+stopifnot(all(is.na(sL@x)), ## not yet:  all(is.na(sL)),
           !any(sL, na.rm=TRUE), all(!sL, na.rm=TRUE),
           validObject(Matrix(c(NA,0), 4, 3, byrow = TRUE)),
-          validObject(Matrix(c(NA,0), 4, 4)),
-          isValid(Matrix(c(NA,0,0,0), 4, 4), "sparseMatrix"))
+          validObject(Matrix(c(NA,0), 4, 4)))
+stopifnotValid(Matrix(c(NA,0,0,0), 4, 4), "sparseMatrix")
 I <- i1 <- I1 <- Diagonal(1)
+## TODO? stopifnot(identical(I, Matrix(1, sparse=TRUE))) # doDiag=TRUE default
 I1[1,1] <- i1[1, ] <- I [ ,1] <- NA
 stopifnot(identical3(I,i1,I1))
 image(d4) # gave infinite recursion
 
+## Steve Walker, Mar 12, 2014:
+n <- 7
+(M <- triu(Matrix(seq_len(n^2), n, sparse=TRUE)))
+im <- image(M) # should be an n-by-n image plot, but is not
+stopifnot(n == diff(sort(im$y.limits)))
+## ylimits were too small (by 1 on each side)
+
 assertError( Matrix(factor(letters)) )
-mlp <- matrix(.leap.seconds)## 24 x 1 numeric matrix
+n.lsec <- length(.leap.seconds)# 27 (2017-07)
+mlp <- matrix(.leap.seconds)## 27 x 1 numeric matrix
 Mlp <- Matrix(.leap.seconds)
+stopifnot(identical(dim(Mlp), c(n.lsec, 1L)))
 assert.EQ.mat(Mlp, mlp)
+.Leap.seconds <- as.POSIXlt(.leap.seconds)
 if(FALSE) { ## TODO -- once R itself does better ...
-    .Leap.seconds <- as.POSIXlt(.leap.seconds)
-    mLp <- matrix(.Leap.seconds)##  24 x 1  list each of length 24 -- yuck!!!
+    mLp <- matrix(.Leap.seconds)##  11 x 1 list of (Numeric,Int.,Char.) each of length 27 -- yuck!!!
     MLp <- Matrix(.Leap.seconds)## --> error (for now)
 }
 
 E <- rep(c(TRUE,NA,TRUE), length=8)
 F <- new("nsparseVector", length = 8L, i = c(2L, 5L, 8L))
 e <- as(E, "sparseVector"); f <- as(F,"lsparseVector")
+Fv <- as.vector(F, "any") # failed in Matrix <= 1.2.0, and base::as.vector(.) failed too:
 stopifnot(E | as.vector(F), identical(E | F, F | E),
-          all(e | f), all(E | F)) # <- failed  Ops.spv.spv
+	  all(e | f), all(E | F), # <- failed  Ops.spv.spv
+	  identical(Fv, base::as.vector(F)),
+	  is.logical(Fv), which(Fv) == c(2,5,8))
+F[-8:-1] # was partly "illegal" (length = 0 is ok; nnz '3' is not)
+F.ineg <- lapply(1:8, function(k) F[-8:-k])
+F.pos <-  lapply(1:8, function(k) F[seq_len(k-1)])
+F.vec <-  lapply(1:8, function(k) Fv[seq_len(k-1)])
+str(F.vec, vec=8)
+(nT <- vapply(F.vec, sum, 1L)) # == 0 0  1 1 1  2 2 2
+str(whichT <- lapply(F.vec, which))
+i.lengths <- function(L) vapply(L, function(.) length(.@i), 1L)
+stopifnot(identical(lengths(F.vec), 0:7)
+         ,
+          identical(lengths(F.ineg), 0:7)
+         ,
+          identical(lengths(F.pos),  0:7)
+         ,
+          identical(i.lengths(F.pos),  nT)
+         ,
+          identical(i.lengths(F.ineg), nT) # failed before 2018-03-19
+         ,
+          identical(lapply(F.pos, slot, "i"), whichT)
+         ,
+          identical(lapply(F.ineg, slot, "i"), whichT) # failed before 2018-03-19
+)
+## Here, sparseVector '[' is really wrong:
+SV <- new("nsparseVector", length = 30L,
+          i = c(1L, 8L, 9L, 12L, 13L, 18L, 21L, 22L))
+NI <- -c(1:5, 7:10, 12:16, 18:27, 29,30)
+sv <- SV[1:14]; ni <- -(1:14)[-c(6,11)] # smaller example
+selectMethod("[", c("nsparseVector","index","missing","missing"))# the culprit
+if(FALSE)
+  trace("[", browser, signature=c("nsparseVector","index","missing","missing"))
+stopifnot(
+    SV[NI] == as.vector(SV)[NI] ## badly failed before 2018-03-19
+    ,
+    sv[ni] == as.vector(sv)[ni] ## ditto
+)
+if(FALSE)
+    untrace("[", signature=c("nsparseVector","index","missing","missing"))
 
 dT <- new("dgTMatrix",
 	  i = c(1:2,1:2), j=rep(1:2, each=2), Dim = c(4L, 4L), x = c(1, 1, NA, 2))
@@ -86,24 +161,28 @@ isValid(lc <- c1 > c2,"lgCMatrix")
 isValid(lt <- dT > dt,"lgCMatrix")
 stopifnot(identical(lc,lt))
 
+## Versions of Diagonal()
+dD <- Diagonal(x = 5:1)
+sD <- .symDiagonal(5, 5:1)
+tD <- .trDiagonal (5, 5:1) # x=<integer> had failed for both
+stopifnot(all(sD == dD)
+        , all(tD == dD)
+        , identical(sD, .symDiagonal(x = 5:1))
+        , identical(tD, .sparseDiagonal(x=5:1))
+        , identical(tD, .trDiagonal (x = 5:1)))# 'n' now has default
+
 M <- Diagonal(4); M[1,2] <- 2 ; M
 cM <- crossprod(M) # >> as_cholmod_l_triplet(): could not reallocate for internal diagU2N()
 stopifnot(identical(cM, tcrossprod(t(M))))
 
-if(doExtras) { ## formerly in MM-only's ./AAA_latest.R
-    ## 2010-11-29 --- prompted by BDR:
-    mlp <- matrix(.leap.seconds)## 24 x 1 numeric matrix
-    Mlp <- Matrix(.leap.seconds)
-    assert.EQ.mat(Mlp, mlp)
+S.na <- spMatrix(3, 4, c(1,2,3), c(2,3,3), c(NA,1,0))
+(S.na <- S.na - 2 * S.na)
+(L <- S.na != 0)
 
-    S.na <- spMatrix(3, 4, c(1,2,3), c(2,3,3), c(NA,1,0))
-    show(S.na <- S.na - 2 * S.na)
-    show(L <- S.na != 0)
+(Ln0 <- S.na != rep(0, prod(dim(L))))
+.Lm0 <- S.na != Matrix(0, 3, 4)
+stopifnot(Q.eq(L, Ln0), identical(Ln0, .Lm0));  rm(Ln0, .Lm0)
 
-    M0 <- Matrix(0, 3, 4)
-    show(Ln0 <- S.na != rep(0, prod(dim(L))))
-    stopifnot(Q.eq(L, Ln0), identical(Ln0, M0 != S.na))
-}## (doExtras) only
 
 ### Unit-diagonal and unitriangular  {methods need diagU2N() or similar}
 I <- Diagonal(3)
@@ -123,7 +202,7 @@ stopifnot(
     identical3(lC, C | FALSE, FALSE | C),
     TRUE)
 I[,1] <- NA; I[2,2] <- NA ; I[3,] <- NaN
-stopifnot(isValid(I, "sparseMatrix"))
+stopifnotValid(I, "sparseMatrix")
 I # gave error in printSpMatrix() - because of R bug in format.info()
 
 L <- spMatrix(9, 30, i = rep(1:9, 3), 1:27, (1:27) %% 4 != 1)
@@ -131,7 +210,7 @@ M <- drop0(crossprod(L))
 diag(M) <- diag(M) + 5 # to make it pos.def.
 M. <- M[1:12,1:12] # small ex
 N3 <- as(Matrix(upper.tri(diag(3))), "nMatrix")
-isValid(bdN <- bdiag(N3, N3),"nsparseMatrix")
+stopifnotValid(bdN <- bdiag(N3, N3),"nsparseMatrix")
 
 stopifnot(identical(L, L == TRUE), ## used to give infinite recursion
           all(drop0((0 - L) != 0) == drop0(L)))
@@ -162,6 +241,45 @@ stopifnot(isSymmetric(M), isSymmetric(M.),
 	  all.equal(M., M2, tolerance =0)
 	  )
 
+Filter(function(.) inherits(get(.), "symmetricMatrix"), ls())
+## [1] "cM" "M"  "M." "M2" "o4" "sc"
+tt <- as(kronecker(cM, Diagonal(x = c(10,1))), "symmetricMatrix")
+dimnames(tt) <- list(NULL, cn <- letters[1:ncol(tt)])
+stopifnotValid(tt, "dsTMatrix")
+(cc <- as(tt, "CsparseMatrix")) # shows *symmetric* dimnames
+stopifnot(identical3(  cc @Dimnames,   tt @Dimnames, list(NULL, cn)),
+	  identical3(t(cc)@Dimnames, t(tt)@Dimnames, list(cn, NULL)),
+	  identical3(dimnames(cc), dimnames(tt), list(cn, cn)))# now symmetric !
+
+stopifnot(identical3(dimnames(cc),
+                     dimnames(as(cc, "generalMatrix")), ## should fixup dimnames to *symmetric*
+                     dimnames(as(tt, "generalMatrix"))))
+## -->  .Call(Csparse_symmetric_to_general, from)
+
+mat <- as(cc, "matrix") ## --> should fixup dimnames to *symmetric*
+mat # should print *symmetric* dimnames
+stopifnot(identical3(dimnames(cc), dimnames(mat), dimnames(as(tt, "matrix"))))
+
+selectMethod(coerce, c("dsCMatrix", "denseMatrix"))
+dmat <- as(cc, "denseMatrix") ## --> gave Error (!!) in Matrix 1.1-5
+stopifnot(identical3(tt@Dimnames, dmat@Dimnames, list(NULL, cn)))
+dmat # should print *symmetric* dimnames (not modifying dmat as it did intermittently)
+stopifnot(identical(dmat@Dimnames, list(NULL, cn)))
+ttdm <- as(tt, "denseMatrix")
+stopifnot(all.equal(dmat, ttdm),
+          ## ^^^^^^ not identical(): 'x' slot differs, as only "U" is needed
+          identical(as(dmat, "dspMatrix"), as(ttdm, "dspMatrix")),
+          identical(dimnames(cc), dimnames(dmat)),
+          ## coercing back should give original :
+	  identical(cc,              as(dmat, "sparseMatrix")),
+	  identical(uniqTsparse(tt), as(ttdm, "TsparseMatrix")))
+
+## MM: now *if* cc is "truly symmetric", these dimnames should be, too:
+d5 <- cn[1:5]; dnm5 <- list(d5,d5)
+stopifnot(identical(dimnames( cc  [1:5, 1:5]), dnm5),
+          identical(dimnames(t(cc)[1:5, 1:5]), dnm5))
+
+
 ## large sparse ones: these now directly "go sparse":
 str(m0 <- Matrix(0,     nrow=100, ncol = 1000))
 str(l0 <- Matrix(FALSE, nrow=100, ncol = 200))
@@ -183,15 +301,100 @@ stopifnot(identical(x*D, (Dx <- D*x)),
 	  identical(Dx, local({d <- D; d@x <- d@x * x; d})))
 
 Lrg <- new("dgTMatrix", Dim = c(n,n))
+l0 <- as(as(Lrg, "lMatrix"), "lgCMatrix")
+d0 <- as(l0, "dgCMatrix")
+
+if(FALSE) { #_____________________ FIXME: Should use cholmod_l_*() everywhere (?)____
+## problem in  Csparse_to_dense :
+dl0 <- as(l0, "denseMatrix")
+dd0 <- as(d0, "denseMatrix")
+## currently, both give --- Error in asMethod(object) :
+##   Cholmod error 'problem too large' at file ../Core/cholmod_dense.c, line 105
+##--> And there it is 'Int_max' ==> ../src/CHOLMOD/Include/cholmod_internal.h
+## defines 'Int_max'  and does that depending of "mode", and
+## MM thinks we should use the "DLONG" mode now -- for 64-bit integers!
+##  ==> Then Int_max := SuiteSparse_long_max := LONG_MAX
+## (the latter from ../src/SuiteSparse_config/SuiteSparse_config.h )
+## ==> use cholmod_l_<foo> instead of cholmod_<foo> in *many places*
+##
+## check they are ok
+stopifnot(identical(dim(dl0), c(n,n)), identical(dim(dd0), c(n,n)),
+          !any(dl0), all(dd0 == 0))
+rm(dl0, dd0)# too large to keep in memory and pass to checkMatrix()
+}
+
 diag(Lrg[2:9,1:8]) <- 1:8
 ## ==:  Lrg[2:9,1:8] <- `diag<-`(Lrg[2:9,1:8], 1:8)
-e1 <- try(Lrg == Lrg) # error message almost ok
-e2 <- try(!Lrg) # error message was "bad", now perfect
+e1 <- try(Lrg == Lrg) # ==> Cholmod error 'problem too large' at file ../Core/cholmod_dense.c, line 105
+## (error message almost ok)
+
+(memGB <- Sys.memGB("MemFree")) # from test-tools-1.R
+system.time( # ~10 sec.                            __vv__
+    e2 <- if(doExtras && is.finite(memGB) && memGB > 30) { # need around 18 GB
+              try(!Lrg) # now *works* on 64-bit machines with enough RAM
+              ## and immediately errors if LONG_VECTORs are not available
+    })
+str(e2) # error, NULL or "worked" (=> 50000 x 50000 lgeMatrix)
 ina <- is.na(Lrg)# "all FALSE"
 stopifnot(grep("too large", e1) == 1,
-          grep("too large", e2) == 1,
+	  if(inherits(e2, "try-error")) grep("too large", e2) == 1
+	  else is.null(e2) || length(e2@x) == n^2,
           !any(ina))# <- gave warning previously
 stopifnot(suppressWarnings(any(Lrg)))# (double -> logical  warning)
+rm(e2)# too large...
+
+RNGversion("3.6.0")# future proof
+if(doExtras && is.finite(memGB) && memGB > 49) withAutoprint({
+    cat("computing SM .. \n")
+    showSys.time(m <- matrix(0, 3e6, 1024))
+    ##  user  system elapsed
+    ## 2.475  10.688  13.196  (faster in past ??)
+    set.seed(1); inot0 <- unique(sort(c(1, length(m), sample(length(m), 20))))
+    ai0 <- arrayInd(inot0, .dim=dim(m), useNames=FALSE)
+    showSys.time(m[inot0] <- 1:22)
+    ##  user  system elapsed
+    ## 5.931  11.184  17.162
+    showSys.time(SM  <- as(m, "sparseMatrix")) # ~ 8 sec
+    ## gave 'Error in asMethod(object) : negative length vectors are not allowed'
+    ## now works - via C  matrix_to_Csparse()
+    showSys.time(n0.m <- c(m) != 0) # logical (full, base R) matrix, 12 GB
+    ##   user  system elapsed
+    ## 14.901  10.789  25.776
+    try( ## _FIXME_ in R: Error ... long vectors not supported yet
+        in0.m <- which(n0.m)
+    )
+    ## DONE: now very fast! [previously did coerce the whole matrix to dense first !]
+    subS <- SM[inot0]
+    selectMethod("[", c("dgCMatrix","numeric","missing","missing"))# -> .M.vectorSub(x,i)
+    ## Directly via arrayInd(), is *FAST*:
+    subSij <- SM[ai0]
+    stopifnot(subS == 1:22, identical(subS, subSij))
+    cat(" [Ok]\n")
+    rm(m)
+    str(SM)
+    ## checking SM:
+    TM <- as(SM, "TsparseMatrix")
+    stopifnot(as.matrix(summary(TM)) == cbind(ai0, 1:22))
+    ## cleanup:
+    rm(SM, TM)
+})
+
+## Constructing *packed* dense symmetric (dsp*) | triangular (dtp*) Matrices:
+if(doExtras && is.finite(memGB) && memGB > 35) withAutoprint({
+    m <- as.integer(2^16) ## = 65536
+    showSys.time(x <- rep(as.numeric(1:100), length.out=m*(m+1)/2))
+    ##  user  system elapsed
+    ## 6.028   8.964  15.074
+    gc()
+    object.size(x) # 17'180'131'368 bytes: ~ 17 GB
+    mat <- new("dspMatrix", x = x, Dim = c(m, m)) # failed with
+    ## long vectors not supported yet: ../../src/include/Rinlinedfuns.h:...
+    validObject(mat)
+    mat <- new("dtpMatrix", x = x, Dim = c(m, m)) # failed .......
+    validObject(mat)
+    ## cleanup
+    rm(mat)
+})
 
 ## with dimnames:
 v <- c(a=1, b=2:3)
@@ -201,13 +404,44 @@ stopifnot(identical(dimnames(m), list(c("a", "b1", "b2"), NULL)),
 	  identical(M, as(m, "dgeMatrix")),
 	  identical(dimnames(M), dimnames(m)))
 
+## dimnames(.) of symmpart() / skewpart() :
+ns <- c("symmpart", "skewpart", "forceSymmetric")
+symFUNs <- setNames(lapply(ns, get), ns); rm(ns)
+chkSS <- function(m) {
+  r <- lapply(symFUNs, function(fn) fn(m))
+  m0 <- as(m, "matrix")
+  r0 <- lapply(symFUNs, function(fn) fn(m0))
+  stopifnotValid(fS  <- r [["forceSymmetric"]], "symmetricMatrix")
+  stopifnotValid(fS0 <- r0[["forceSymmetric"]], "symmetricMatrix")
+  dnms <- dimnames(m)
+  d.sy <- dimnames(r[["symmpart"]])
+  id <- if(is.null(dnms[[2]]) && !is.null(dnms[[1]])) 1 else 2
+  stopifnot(identical(d.sy, dnms[c(id,id)]),
+	    identical(d.sy, dimnames(r [["skewpart"]])),
+	    identical(d.sy, dimnames(r0[["skewpart"]])),
+	    all(m  == with(r,  symmpart + skewpart)),
+	    all(m0 == with(r0, symmpart + skewpart)),
+	    identical(dS <- dimnames(fS), dimnames(fS0)),
+	    identical(dS[1], dS[2]))
+}
+cat(sprintf("chkSS() {valid %s} for a list of matrices:\n",
+            paste(paste0(names(symFUNs), "()"), collapse=", ")))
+for(m in list(Matrix(1:4, 2,2), Matrix(c(0, rep(1:0, 3),0:1), 3,3))) {
+    cat("\n---\nm:\n"); show(m)
+    chkSS(m)
+    dn <- list(row = paste0("r", 1:nrow(m)), col = paste0("var.", 1:ncol(m)))
+    dimnames(m) <- dn		; chkSS(m)
+    colnames(m) <- NULL		; chkSS(m)
+    dimnames(m) <- unname(dn)	; chkSS(m)
+}
+
 m. <- matrix(c(0, 0, 2:0), 3, 5)
 dimnames(m.) <- list(LETTERS[1:3], letters[1:5])
 (m0 <- m <- Matrix(m.))
 m@Dimnames[[2]] <- m@Dimnames[[1]]
 ## not valid anymore:
 (val <- validObject(m, test=TRUE)); stopifnot(is.character(val))
-dm <- as(m0, "denseMatrix")
+dm <- as(m0, "denseMatrix"); rm(m)
 stopifnot(all.equal(rcond(dm), rcond(m.), tolerance = 1e-14),
 	  ##^^^^^^^ dm and m. are both dense, interestingly small differences
 	  ## show in at least one case of optimized BLAS
@@ -215,7 +449,9 @@ stopifnot(all.equal(rcond(dm), rcond(m.), tolerance = 1e-14),
           ## show(<dgRMatrix>) had revealed a bug in C:
           identical(capture.output(show(as(m0, "RsparseMatrix")))[-(1:2)],
                     gsub("0", ".",  capture.output(show(m.))[-1])))
-rm(m)
+m.1 <- m.; dimnames(m.1) <- list(row=NULL, col=NULL)
+M.1 <- Matrix(m.1, sparse=TRUE)
+show(M.1)# had bug in .formatSparseSimple()
 
 ###--  Sparse Triangular :
 
@@ -230,25 +466,27 @@ stopifnot(class(t5) == "dtCMatrix",
 ## Maybe move to R once 'Matrix' is recommended
 sm <- selectMethod(coerce, c("dgCMatrix", "triangularMatrix"), verbose=TRUE)
 stopifnot(identical(sm(g5), t5))
-
+dimnames(t5) <- list(row=paste0("r",1:5), col=paste0("C.",1:5))
+s5 <- symmpart(t5) # gave an error
 
 (t1 <- new("dtTMatrix", x= c(3,7), i= 0:1, j=3:2,
            Dim= as.integer(c(4,4))))
 ## Diagonal  o  Sparse
 I4 <- Diagonal(4)
 D4 <- Diagonal(4, x=1:4)
+validObject(t1)
 validObject(t2  <-   t1  + I4)
 validObject(tt2 <- t(t1) + I4)
 validObject(t1c <- as(t1, "CsparseMatrix"))
 validObject(t2c <- as(t2, "CsparseMatrix"))
-stopifnot(validObject(t1),
-          isValid(2 * I4, "diagonalMatrix"),
-          isValid(D4 * 3, "diagonalMatrix"),
-          isValid(I4 / 5, "diagonalMatrix"),
-          isValid(D4 / 2, "diagonalMatrix"),
-          identical(t1, t(t(t1))),
+stopifnotValid(2 * I4, "diagonalMatrix")
+stopifnotValid(D4 * 3, "diagonalMatrix")
+stopifnotValid(I4 / 5, "diagonalMatrix")
+stopifnotValid(D4 / 2, "diagonalMatrix")
+stopifnotValid(t1c + I4,"triangularMatrix")
+stopifnotValid(t2c + I4,"triangularMatrix")
+stopifnot(identical(t1, t(t(t1))),
           identical(t1c, t(t(t1c))),
-          isValid(t1c + I4,"triangularMatrix"), isValid(t2c + I4,"triangularMatrix"),
           c(class(t2), class(t1c), class(t2c), class(tt2)) == "dtCMatrix",
           identical(t(tt2), t2))
 assert.EQ.mat(t1, as(t1c, "matrix"))
@@ -268,6 +506,13 @@ stopifnot(is(Lg1, "diagonalMatrix"), is(D4m, "diagonalMatrix"),
           identical3(Lg1, Matrix(nnLg), as(nnLg, "diagonalMatrix")),
           all(Lg1 != (!Lg1)))
 
+## tri[lu](<diagonal>)
+td3 <- triu(diag(3)); stopifnot(is(td3, "triangularMatrix"), td3@uplo == "U")
+Ld3 <- tril(diag(3)); stopifnot(is(Ld3, "triangularMatrix"), Ld3@uplo == "L")
+## the latter did fail till 2014-12-20
+D3 <- Diagonal(3)
+stopifnot(identical3(D3, tril(D3), triu(D3)))
+## methods were missing
 
 ## as(<diag>, <anything>) :
 str(cls <- names(getClass("Matrix")@subclasses))# all Matrix classes
@@ -355,14 +600,14 @@ stopifnot(validObject(U), ## had a case where solve(U) modified U !
 	  identical(U, diagN2U(drop0(U.))))
 
 ## <sparse> o <numeric> (of length > 1):
-stopifnot(isValid(tm <- tu * 1:8, "sparseMatrix"),
-          identical4(tm, cu * 1:8, 1:8 * cu, 1:8 * tu))
+stopifnotValid(tm <- tu * 1:8, "sparseMatrix")
+stopifnot(identical4(tm, cu * 1:8, 1:8 * cu, 1:8 * tu))
 
 cu[1,2] <- tu[1,2] <- NA
 mu <- as(tu,"matrix")
-stopifnot(isValid(cu, "CsparseMatrix"), isValid(cu, "triangularMatrix"),
-          isValid(tu, "TsparseMatrix"), isValid(tu, "triangularMatrix"),
-          identical(cu * 1:8, tu * 1:8), # but are no longer triangular
+stopifnotValid(cu, "CsparseMatrix"); stopifnotValid(cu, "triangularMatrix")
+stopifnotValid(tu, "TsparseMatrix"); stopifnotValid(tu, "triangularMatrix")
+stopifnot(identical(cu * 1:8, tu * 1:8), # but are no longer triangular
           identical(cu > .1, as(tu > .1, "CsparseMatrix")),
           all(cu >= 0, na.rm=TRUE), !all(cu >= 1), is.na(all(tu >= 0)),
           ## Csparse_drop: preserves triangularity incl diag="U"
@@ -380,7 +625,7 @@ stopifnot(all(ina == is.na(cu)),
 set.seed(7)
 xx <- rpois(10, 50)
 Samp <- function(n,size) sample(n, size, replace=TRUE)
-Tn <- sparseMatrix(i=Samp(8, 50), j=Samp(9,50), x=xx, giveCsparse=FALSE)
+Tn <- sparseMatrix(i=Samp(8, 50), j=Samp(9,50), x=xx, repr = "T")
 Tn
 stopifnot(xx == Tn@x,
 	  max(xx) < max(Tn), 0 == min(Tn),
@@ -401,9 +646,9 @@ stopifnot(identical3(cu[cu > 1],  tu [tu > 1], mu [mu > 1]),
 	  identical(ttu , tril(ttu)),
 	  identical(t(tu), tril(t(tu)))
           )
-assert.EQ.mat(triu(cu),   as.matrix(triu(as.matrix(cu))))
+assert.EQ.mat(triu(cu),   .asmatrix(triu(.asmatrix(cu))))
 for(k in -1:1)
-    assert.EQ.mat(tril(cu,k), as.matrix(tril(as.matrix(cu),k)))
+    assert.EQ.mat(tril(cu,k), .asmatrix(tril(.asmatrix(cu),k)))
 
 (dtr <- Matrix(local({m <- diag(2); m[1,2] <- 3;m})))
 identical(dtr, triu(dtr))
@@ -425,8 +670,8 @@ set.seed(123)
 mm. <- mm <- Matrix(rnorm(500 * 150), nc = 150)
 stopifnot(validObject(mm))
 xpx <- crossprod(mm)
-stopifnot(identical(mm, mm.),# once upon a time, mm was altered by crossprod()
-          isValid(xpx, "dpoMatrix"))
+stopifnot(identical(mm, mm.))# once upon a time, mm was altered by crossprod()
+stopifnotValid(xpx, "dpoMatrix")
 str(mm) # 'dge*"
 str(xpx)# 'dpo*"
 xpy <- crossprod(mm, rnorm(500))
@@ -494,12 +739,27 @@ system.time(IT3 <- solve(T3k))# incredibly fast
 I. <- drop0(zapsmall(IT3 %*% T3k))
 I.. <- diagN2U(I.)
 I <- Diagonal(5^5)
-stopifnot(isValid(IT3, "dtCMatrix"),
-          ## something like the equivalent of  all(I. == Diagonal(3125)) :
+stopifnotValid(IT3, "dtCMatrix")
+stopifnot(## something like the equivalent of  all(I. == Diagonal(3125)) :
           identical(as(I., "diagonalMatrix"), I),
           identical(as(I..,"diagonalMatrix"), I)
           )
 
+## printSpMatrix() ;  "suppressing (columns | rows) .." {and do it correctly!}
+IT3
+op0 <- options(width = 70, max.print = 1000)
+T125[-(1:50),] ## suppression ... is it correctly done?
+
+## Still buggy -- FIXME: see ../TODO --- even if we'd require max.print >= 5 or so
+for(mm in 1:21) {
+    options(max.print=mm)
+    cat("----------\n\nmax.print=",mm,":\n", sep="")
+    cat("\n>> U:   ") ; show(U)
+    cat("\n>> slp: ") ; show(slp)
+}
+
+options(op0)# revert to max.print = 1000
+
 ###-- row- and column operations  {was ./rowcolOps.R }
 
 set.seed(321)
@@ -509,10 +769,14 @@ m.m <- as(m1k, "matrix")
 stopifnot(all.equal(colMeans(m1k), colMeans(m.m)),
           all.equal(colSums (m1k), colSums (m.m)),
           all.equal(rowMeans(m1k), rowMeans(m.m)),
-          all.equal(rowSums (m1k), rowSums (m.m)))
+          all.equal(rowSums (m1k), rowSums (m.m)),
+          all.equal(colMeans(m1k, na.rm=TRUE), colMeans(m.m, na.rm=TRUE)),
+          all.equal(colSums (m1k, na.rm=TRUE), colSums (m.m, na.rm=TRUE)),
+          all.equal(rowMeans(m1k, na.rm=TRUE), rowMeans(m.m, na.rm=TRUE)),
+          all.equal(rowSums (m1k, na.rm=TRUE), rowSums (m.m, na.rm=TRUE)) )
 
 ###-- kronecker for nonsparse uses Matrix(.):
-stopifnot(isValid(kr <- kronecker(m1, m6), "Matrix"))
+stopifnotValid(kr <- kronecker(m1, m6), "Matrix")
 assert.EQ.mat(kr,
               kronecker(as(m1, "matrix"),
                         as(m6, "matrix")), tol = 0)
@@ -522,7 +786,7 @@ assert.EQ.mat(kr,
 kt2 <- kronecker(t1c, cu)
 stopifnot(identical(Matrix:::uniq(kt1), Matrix:::uniq(kt2)))
 ## but kt1 and kt2, both "dgT" are different since entries are not ordered!
-ktf <- kronecker(as.matrix(t1), as.matrix(tu))
+ktf <- kronecker(.asmatrix(t1), .asmatrix(tu))
 if(FALSE) # FIXME? our kronecker treats "0 * NA" as "0" for structural-0
 assert.EQ.mat(kt2, ktf, tol= 0)
 (cs1 <- colSums(kt1))
@@ -547,8 +811,10 @@ for(M in list(kt1, nt1, ng1, dg1, lt1, nt1)) {
 	cs. <- colSums(M, na.rm = na.rm, sparseResult = TRUE)
 	rs  <- rowSums(M, na.rm = na.rm)
 	rs. <- rowSums(M, na.rm = na.rm, sparseResult = TRUE)
-	stopifnot(isValid(cs., "sparseVector"), identical(cs, as(cs., "vector")),
-                  isValid(rs., "sparseVector"), identical(rs, as(rs., "vector")),
+	stopifnotValid(cs., "sparseVector")
+	stopifnotValid(rs., "sparseVector")
+	stopifnot(identical(cs, as(cs., "vector")),
+		  identical(rs, as(rs., "vector")),
 		  {eq <- cs == colSums(m, na.rm = na.rm) ; ineq <- is.na(eq)
 		   all(ineq | eq) && identical(ineq, is.na(cs)) },
 		  {eq <- rs == rowSums(m, na.rm = na.rm) ; ineq <- is.na(eq)
@@ -584,7 +850,7 @@ stopifnot(identical(N, as(L.,"nMatrix")),
 	  identical(kronecker(	  c(1,0), M),
 		    kronecker(cbind(1:0), M)))
 assert.EQ.mat(kronecker(M,	      c(1,0,0)),
-	      kronecker(as.matrix(M), c(1,0,0)))
+	      kronecker(.asmatrix(M), c(1,0,0)))
 
 ## coercion from "dpo" or "dsy"
 xx <- as(xpx, "dsyMatrix")
@@ -610,16 +876,17 @@ stopifnot(as.numeric(nn[nn != ""]) == m[m != 0],
 assert.EQ.mat(tM, tm, tol=0)
 assert.EQ.mat(gC, m,  tol=0)
 assert.EQ.mat(mT, m,  tol=0)
-stopifnot(isValid(mM, "dsCMatrix"), isValid(tM, "dtCMatrix")
-	  , identical(mT, as(mM, "TsparseMatrix"))
+stopifnotValid(mM, "dsCMatrix")
+stopifnotValid(tM, "dtCMatrix")
+stopifnot(identical(mT, as(mM, "TsparseMatrix"))
 	  , identical(gC, as(mM, "generalMatrix"))
 	  ## coercions	general <-> symmetric
 	  , identical(as(as(mM, "generalMatrix"), "symmetricMatrix"), mM)
 	  , identical(as(as(mM, "dgTMatrix"),     "symmetricMatrix"), mT)
 	  , identical(as(as(tM, "generalMatrix"),"triangularMatrix"), tM)
           , identical(tM + Diagonal(8), tMD <- Diagonal(8) + tM)
-          , isValid(tMD, "dtCMatrix")
 	  )
+stopifnotValid(tMD, "dtCMatrix")
 eM <- eigen(mM) # works thanks to base::as.matrix hack in ../R/zzz.R
 stopifnot(all.equal(eM$values,
                 { v <- c(162.462112512353, 30.0665927567458)
@@ -685,8 +952,8 @@ as(nC,"denseMatrix")
 (nkt <- as(as(as(kt1, "generalMatrix"), "CsparseMatrix"), "ngCMatrix"))# ok
 dkt <- as(nkt, "denseMatrix")
 (clt <- crossprod(nkt))
-stopifnot(isValid(nkt, "ngCMatrix"),
-          isValid(clt, "nsCMatrix"))
+stopifnotValid(nkt, "ngCMatrix")
+stopifnotValid(clt, "nsCMatrix")
 suppressWarnings(crossprod(clt)) ## warning "crossprod() of symmetric ..."
 
 ## a Csparse with *repeated* entry is not valid!
@@ -715,12 +982,12 @@ ij <- function(a) a@i + ncol(a) * a@j
 stopifnot(all(ij(A) %in% ij(B)))
 
 l3 <- upper.tri(matrix(,3,3))
-stopifnot(isValid(c3 <- as(l3, "CsparseMatrix"), "CsparseMatrix"),# lgC
-          is(c3, "lMatrix"))
+validObject(c3 <- as(l3, "CsparseMatrix"))
+stopifnotValid(c3, "lMatrix")# lgC
 (M <- Matrix(l3))
-stopifnot(isValid(M, "ltCMatrix"),
-          isValid(M2 <- M %x% M, "triangularMatrix"), # is "dtT" (why not "dtC" ?)
-          dim(M2) == c(9,9), identical(M2, kronecker(M,M)))
+stopifnotValid(M, "ltCMatrix")
+stopifnotValid(M2 <- M %x% M, "triangularMatrix") # is "dtT" (why not "dtC" ?)
+stopifnot(dim(M2) == c(9,9), identical(M2, kronecker(M,M)))
 M3 <- M %x% M2 #ok
 (cM3 <- colSums(M3, sparse=TRUE))
 identical(as.vector(cM3),
@@ -728,21 +995,21 @@ identical(as.vector(cM3),
 M. <- M2 %x% M # gave infinite recursion
 
 ## diagonal, sparse & interactions
-stopifnot(isValid(as(Diagonal(3), "TsparseMatrix"), "TsparseMatrix"),
-          isValid(X <- Diagonal(7) + 1.5 * tM[1:7,1:7], "sparseMatrix"),
-          isValid(X, "triangularMatrix"),
-          isValid(XX <- X - chol(crossprod(X)), "triangularMatrix"))
+stopifnotValid(as(Diagonal(3), "TsparseMatrix"), "TsparseMatrix")
+stopifnotValid(X <- Diagonal(7) + 1.5 * tM[1:7,1:7], "sparseMatrix")
+stopifnotValid(X, "triangularMatrix")
+stopifnotValid(XX <- X - chol(crossprod(X)), "triangularMatrix")
 X
 XX
 XX <- as(drop0(XX), "dsCMatrix")
-stopifnot(identical(XX, Matrix(0, nrow(X), ncol(X))))
+stopifnot(identical(XX, Matrix(0, nrow(X), ncol(X), doDiag=FALSE)))
 
 M <- Matrix(m., sparse = FALSE)
 (sM <- Matrix(m.))
 class(dlM <- M >= 1)
-stopifnot(identical(dlM, !(M < 1)),
-	  isValid(sM, "sparseMatrix"),
-	  isValid(dlM, "denseMatrix"))
+stopifnot(identical(dlM, !(M < 1)))
+stopifnotValid(sM, "sparseMatrix")
+stopifnotValid(dlM, "denseMatrix")
 (lM  <- as(dlM, "sparseMatrix"))
 lM2 <- as(dlM, "CsparseMatrix") #-> now ok
 lM0 <- Matrix:::as_Csparse(dlM)
@@ -772,8 +1039,8 @@ assert.EQ.mat( crossprod(v, nt),  crossprod(v,mnt))
 assert.EQ.mat(tcrossprod(v, nt), tcrossprod(v,mnt))
 assert.EQ.mat(tcrossprod(nt, m), tcrossprod(mnt, m))
 ##
-stopifnot(isValid(ms, "dsTMatrix"),
-          as(ms0,"matrix") == as(ll, "matrix"), # coercing num |-> log
+stopifnotValid(ms, "dsTMatrix")
+stopifnot(as(ms0,"matrix") == as(ll, "matrix"), # coercing num |-> log
 	  as(lt, "matrix") == as(ll, "matrix"),
 	  identical(ms, as(ll, "dMatrix")),
 	  identical4(as(ll, "CsparseMatrix"), as(cs, "lMatrix"),# lsC*
@@ -802,8 +1069,8 @@ stopifnot(identical(M1, bandSparse(n,m, k=0, diag = list(diag(r1)))),
 s1 <- as(r1, "sparseMatrix") # such that band(s1) is sparse, too
 for(k1 in (-n):m)
     for(k2 in k1:m) {
-        isValid(br1 <- band(r1, k1,k2), "ddenseMatrix")
-        isValid(bs1 <- band(s1, k1,k2), "CsparseMatrix")
+	stopifnotValid(br1 <- band(r1, k1,k2), "ddenseMatrix")
+	stopifnotValid(bs1 <- band(s1, k1,k2), "CsparseMatrix")
         stopifnot(all(r1 == s1))
     }
 
@@ -821,9 +1088,9 @@ validObject(new("isparseVector"))
 R <- sv <- as(D4, "sparseVector")
 ## dim(<sparseVector>) <- (n1,n2)  --> sparse Matrix :
 dim(R) <- dim(D4)
-stopifnot(isValid(sv,"sparseVector"),
-	  isValid(R, "sparseMatrix"),
-	  identical(D4, as(R, "diagonalMatrix")))
+stopifnotValid(sv,"sparseVector")
+stopifnotValid(R, "sparseMatrix")
+stopifnot(identical(D4, as(R, "diagonalMatrix")))
 iv <- c(rep(0, 5), 3, 0,0,7,0,0,0)
 sv <- as(iv, "sparseVector")
 sv. <- as(as.integer(iv), "sparseVector")
@@ -867,9 +1134,15 @@ table(uniC <- sapply(lst[istri], function(.) get(.)@diag == "U"))
 lsUtr <- lst[istri][uniC]
 (di <- sapply(lsUtr, function(.) dim(get(.))))
 ## TODO: use %*%, crossprod(), .. on all those  4 x 4 -- and check "triangular rules"
-cat('Time elapsed: ', (.pt <- proc.time()),'\n') # "stats"
-##
 
+r <- tryCatch(chol2inv(Diagonal(x=1:10), pi=pi), warning=identity)
+stopifnot(grepl("extra argument pi .*chol2inv\\(Diagonal", r$message))
+
+assertError(new("ltrMatrix", Dim = c(2L,2L), x=TRUE))# gave "illegal" object w/o error
+assertError(new("ntrMatrix", Dim = c(2L,2L)))#  dito
+
+
+cat('Time elapsed: ', (.pt <- proc.time()),'\n') # "stats"
 cat("doExtras:",doExtras,"\n")
 if(doExtras) {
     cat("checkMatrix() of all: \n---------\n")
@@ -879,7 +1152,27 @@ if(doExtras) {
 	checkMatrix(.m)
     }
     cat('Time elapsed: ', proc.time() - .pt,'\n') # "stats"
-}
+} 
+## in any case, test
+d4d.2 <- Matrix:::.dense2C(!!d4da) ## <<- did wrongly make dimnames symmetric
+l4da <- as(d4da, "lMatrix")
+assert.EQ.Mat(l4da, as(l4da,"CsparseMatrix"))
+
+dtr <- tr4 <- triu(Matrix(1:16, 4,4))
+dtr@x[Matrix:::indTri(4, upper=FALSE, diag=FALSE)] <- 100*(-3:2)
+stopifnot(all.equal(dtr, tr4), # because are same *as* simple matrices
+	  dtr@x[1:4] == c(1, -(3:1)*100),
+	  range(tr4) == c(0,16),
+	  range(dtr) == c(0,16)) # <- failed
+
+##  new("nsyMatrix") + new("lgeMatrix") # failed
+cln <- sort(outer(c("l","n"), paste0(c("ge","sy"), "Matrix"), paste0))
+dim(c.c <- as.matrix(expand.grid(cln, cln, KEEP.OUT.ATTRS=FALSE))) # 16 x 2
+## clTry <- function(expr) class(tryCatch(expr, error=identity))[[1]]
+## '+' [Arith] failed -- now fixed
+cbind(c.c, Res = apply(c.c, 1, function(x) class(new(x[1]) + new(x[2]))))
+## '<' [Compare] works fine
+cbind(c.c, Res = apply(c.c, 1, function(x) class(new(x[1]) < new(x[2]))))
 
 if(!interactive()) warnings()
 
@@ -891,6 +1184,6 @@ sessionInfo()
 c(Matrix = packageDescription("Matrix")$Built)
 if(SysI[["sysname"]] == "Linux" && require("sfsmisc")) local({
     nn <- names(.Sc <- sfsmisc::Sys.cpuinfo())
-    nn <- names(.Sc <- .Sc[nn != "flags"])
-    print(.Sc[grep("\\.[0-9]$", nn, invert=TRUE)])
+    nn <- names(.Sc <- .Sc[!grepl("^flags", nn)])
+    print(.Sc[ grep("\\.[0-9]+$", nn, invert=TRUE) ])
 })
